// This is ImageConvertToRGBA.C
//
// Copyright 1997, 1998     Mark Manca
//


#include "Image.h"



void Image::convertToRGBA() {
	if (_imageType == GL_RGBA) {
		return;
	}

	if (_width == 0 || _height == 0) {
		_imageType = GL_RGBA;
		return;
	}


	void *oldData;
	oldData = _imageData;
	int oldExtraRowWidth = _extraRowWidth;

	_numChannels = 4;
	_allocateImageData();



	int i, j;


	//
	// duplicate a *LOT* of code to make operations go faster. Also, the large
	// amount of code duplication is necessary due to the different data types
	// that an image can be.
	//
	GLbyte *byteColormapPtr = (GLbyte*)_colormapData;
	GLubyte *ubyteColormapPtr = (GLubyte*)_colormapData;
	GLshort *shortColormapPtr = (GLshort*)_colormapData;
	GLushort *ushortColormapPtr = (GLushort*)_colormapData;
	GLint *intColormapPtr = (GLint*)_colormapData;
	GLuint *uintColormapPtr = (GLuint*)_colormapData;
	GLfloat *floatColormapPtr = (GLfloat*)_colormapData;
    
	GLbyte *byteColorPtr;
	GLubyte *ubyteColorPtr;
	GLshort *shortColorPtr;
	GLushort *ushortColorPtr;
	GLint *intColorPtr;
	GLuint *uintColorPtr;
	GLfloat *floatColorPtr;


	if (_imageDataType == GL_BYTE) {
		GLbyte *byteImagePtr = (GLbyte*)_imageData;
		GLbyte *oldByteImagePtr = (GLbyte*)oldData;

		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[*oldByteImagePtr++ * 3];

						*byteImagePtr++ = *byteColorPtr++;
						*byteImagePtr++ = *byteColorPtr++;
						*byteImagePtr++ = *byteColorPtr;
						*byteImagePtr++ = 0x7f;
					}
					byteImagePtr += _extraRowWidth;
					oldByteImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[*oldByteImagePtr++ * 3];

						*byteImagePtr++ = *ubyteColorPtr++;
						*byteImagePtr++ = *ubyteColorPtr++;
						*byteImagePtr++ = *ubyteColorPtr;
						*byteImagePtr++ = 0x7f;
					}
					byteImagePtr += _extraRowWidth;
					oldByteImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[*oldByteImagePtr++ * 3];

						*byteImagePtr++ = (char)*shortColorPtr++;
						*byteImagePtr++ = (char)*shortColorPtr++;
						*byteImagePtr++ = (char)*shortColorPtr;
						*byteImagePtr++ = (char)0x7f;
					}
					byteImagePtr += _extraRowWidth;
					oldByteImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[*oldByteImagePtr++ * 3];

						*byteImagePtr++ = (char)*ushortColorPtr++;
						*byteImagePtr++ = (char)*ushortColorPtr++;
						*byteImagePtr++ = (char)*ushortColorPtr++;
						*byteImagePtr++ = (char)0x7f;
					}
					byteImagePtr += _extraRowWidth;
					oldByteImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[*oldByteImagePtr++ * 3];

						*byteImagePtr++ = *intColorPtr++;
						*byteImagePtr++ = *intColorPtr++;
						*byteImagePtr++ = *intColorPtr++;
						*byteImagePtr++ = 0x7f;
					}
					byteImagePtr += _extraRowWidth;
					oldByteImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[*oldByteImagePtr++ * 3];

						*byteImagePtr++ = *uintColorPtr++;
						*byteImagePtr++ = *uintColorPtr++;
						*byteImagePtr++ = *uintColorPtr++;
						*byteImagePtr++ = 0x7f;
					}
					byteImagePtr += _extraRowWidth;
					oldByteImagePtr += oldExtraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[*oldByteImagePtr++ * 3];

						*byteImagePtr++ = (char)*floatColorPtr++;
						*byteImagePtr++ = (char)*floatColorPtr++;
						*byteImagePtr++ = (char)*floatColorPtr++;
						*byteImagePtr++ = (char)0x7f;
					}
					byteImagePtr += _extraRowWidth;
					oldByteImagePtr += oldExtraRowWidth;
				}
			}
		}
		else {      // _imageType == GL_RGB
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*byteImagePtr++ = *oldByteImagePtr++;
					*byteImagePtr++ = *oldByteImagePtr++;
					*byteImagePtr++ = *oldByteImagePtr++;
					*byteImagePtr++ = 0x7f;
				}
				byteImagePtr += _extraRowWidth;
				oldByteImagePtr += oldExtraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_BYTE) {
		GLubyte *ubyteImagePtr = (GLubyte*)_imageData;
		GLubyte *oldUByteImagePtr = (GLubyte*)oldData;
	
		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[*oldUByteImagePtr++ * 3];

						*ubyteImagePtr++ = *byteColorPtr++ * 2;
						*ubyteImagePtr++ = *byteColorPtr++ * 2;
						*ubyteImagePtr++ = *byteColorPtr * 2;
						*ubyteImagePtr++ = 0xff;
					}
					ubyteImagePtr += _extraRowWidth;
					oldUByteImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[*oldUByteImagePtr++ * 3];

						*ubyteImagePtr++ = *ubyteColorPtr++;
						*ubyteImagePtr++ = *ubyteColorPtr++;
						*ubyteImagePtr++ = *ubyteColorPtr;
						*ubyteImagePtr++ = 0xff;
					}
					ubyteImagePtr += _extraRowWidth;
					oldUByteImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[*oldUByteImagePtr++ * 3];

						*ubyteImagePtr++ = *shortColorPtr++ / 128;
						*ubyteImagePtr++ = *shortColorPtr++ / 128;
						*ubyteImagePtr++ = *shortColorPtr / 128;
						*ubyteImagePtr++ = 0xff;
					}
					ubyteImagePtr += _extraRowWidth;
					oldUByteImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[*oldUByteImagePtr++ * 3];

						*ubyteImagePtr++ = *ushortColorPtr++ / 256;
						*ubyteImagePtr++ = *ushortColorPtr++ / 256;
						*ubyteImagePtr++ = *ushortColorPtr / 256;
						*ubyteImagePtr++ = 0xff;
					}
					ubyteImagePtr += _extraRowWidth;
					oldUByteImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[*oldUByteImagePtr++ * 3];

						*ubyteImagePtr++ = *intColorPtr++ / 8388608;
						*ubyteImagePtr++ = *intColorPtr++ / 8388608;
						*ubyteImagePtr++ = *intColorPtr / 8388608;
						*ubyteImagePtr++ = 0xff;
					}
					ubyteImagePtr += _extraRowWidth;
					oldUByteImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[*oldUByteImagePtr++ * 3];

						*ubyteImagePtr++ = *uintColorPtr++ / 16777216;
						*ubyteImagePtr++ = *uintColorPtr++ / 16777216;
						*ubyteImagePtr++ = *uintColorPtr / 16777216;
						*ubyteImagePtr++ = 0xff;
					}
					ubyteImagePtr += _extraRowWidth;
					oldUByteImagePtr += oldExtraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[*oldUByteImagePtr++ * 3];

						*ubyteImagePtr++ = (unsigned char)(*floatColorPtr++ * 255);
						*ubyteImagePtr++ = (unsigned char)(*floatColorPtr++ * 255);
						*ubyteImagePtr++ = (unsigned char)(*floatColorPtr * 255);
						*ubyteImagePtr++ = (unsigned char)0xff;
					}
					ubyteImagePtr += _extraRowWidth;
					oldUByteImagePtr += oldExtraRowWidth;
				}
			}
		}
		else {      // _imageType == GL_RGB
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ubyteImagePtr++ = *oldUByteImagePtr++;
					*ubyteImagePtr++ = *oldUByteImagePtr++;
					*ubyteImagePtr++ = *oldUByteImagePtr++;
					*ubyteImagePtr++ = 0xff;
				}
				ubyteImagePtr += _extraRowWidth;
				oldUByteImagePtr += oldExtraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_SHORT) {
		GLshort *shortImagePtr = (GLshort*)_imageData;
		GLshort *oldShortImagePtr = (GLshort*)oldData;
		
		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[*oldShortImagePtr++ * 3];

						*shortImagePtr++ = *byteColorPtr++ * 256;
						*shortImagePtr++ = *byteColorPtr++ * 256;
						*shortImagePtr++ = *byteColorPtr * 256;
						*shortImagePtr++ = 0x7fff;
					}
					shortImagePtr += _extraRowWidth;
					oldShortImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[*oldShortImagePtr++ * 3];

						*shortImagePtr++ = *ubyteColorPtr++ * 128;
						*shortImagePtr++ = *ubyteColorPtr++ * 128;
						*shortImagePtr++ = *ubyteColorPtr * 128;
						*shortImagePtr++ = 0x7fff;
					}
					shortImagePtr += _extraRowWidth;
					oldShortImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[*oldShortImagePtr++ * 3];

						*shortImagePtr++ = *shortColorPtr++;
						*shortImagePtr++ = *shortColorPtr++;
						*shortImagePtr++ = *shortColorPtr;
						*shortImagePtr++ = 0x7fff;
					}
					shortImagePtr += _extraRowWidth;
					oldShortImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[*oldShortImagePtr++ * 3];

						*shortImagePtr++ = *ushortColorPtr++ / 2;
						*shortImagePtr++ = *ushortColorPtr++ / 2;
						*shortImagePtr++ = *ushortColorPtr / 2;
						*shortImagePtr++ = 0x7fff;
					}
					shortImagePtr += _extraRowWidth;
					oldShortImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[*oldShortImagePtr++ * 3];

						*shortImagePtr++ = *intColorPtr++ / 65536;
						*shortImagePtr++ = *intColorPtr++ / 65536;
						*shortImagePtr++ = *intColorPtr / 65536;
						*shortImagePtr++ = 0x7fff;
					}
					shortImagePtr += _extraRowWidth;
					oldShortImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[*oldShortImagePtr++ * 3];

						*shortImagePtr++ = *uintColorPtr++ / 131072;
						*shortImagePtr++ = *uintColorPtr++ / 131072;
						*shortImagePtr++ = *uintColorPtr / 131072;
						*shortImagePtr++ = 0x7fff;
					}
					shortImagePtr += _extraRowWidth;
					oldShortImagePtr += oldExtraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[*oldShortImagePtr++ * 3];

						*shortImagePtr++ = (short)(*floatColorPtr++ * 32767);
						*shortImagePtr++ = (short)(*floatColorPtr++ * 32767);
						*shortImagePtr++ = (short)(*floatColorPtr * 32767);
						*shortImagePtr++ = (short)0x7fff;
					}
					shortImagePtr += _extraRowWidth;
					oldShortImagePtr += oldExtraRowWidth;
				}
			}
		}
		else {      // _imageType == GL_RGB
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*shortImagePtr++ = *oldShortImagePtr++;
					*shortImagePtr++ = *oldShortImagePtr++;
					*shortImagePtr++ = *oldShortImagePtr++;
					*shortImagePtr++ = 0x7fff;
				}
				shortImagePtr += _extraRowWidth;
				oldShortImagePtr += oldExtraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_SHORT) {
		GLushort *ushortImagePtr = (GLushort*)_imageData;
		GLushort *oldUShortImagePtr = (GLushort*)oldData;
	
		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[*oldUShortImagePtr++ * 3];

						*ushortImagePtr++ = *byteColorPtr++ * 512;
						*ushortImagePtr++ = *byteColorPtr++ * 512;
						*ushortImagePtr++ = *byteColorPtr * 512;
						*ushortImagePtr++ = 0xffff;
					}
					ushortImagePtr += _extraRowWidth;
					oldUShortImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[*oldUShortImagePtr++ * 3];

						*ushortImagePtr++ = *ubyteColorPtr++ * 256;
						*ushortImagePtr++ = *ubyteColorPtr++ * 256;
						*ushortImagePtr++ = *ubyteColorPtr * 256;
						*ushortImagePtr++ = 0xffff;
					}
					ushortImagePtr += _extraRowWidth;
					oldUShortImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[*oldUShortImagePtr++ * 3];

						*ushortImagePtr++ = *shortColorPtr++ * 2;
						*ushortImagePtr++ = *shortColorPtr++ * 2;
						*ushortImagePtr++ = *shortColorPtr * 2;
						*ushortImagePtr++ = 0xffff;
					}
					ushortImagePtr += _extraRowWidth;
					oldUShortImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[*oldUShortImagePtr++ * 3];

						*ushortImagePtr++ = *ushortColorPtr++;
						*ushortImagePtr++ = *ushortColorPtr++;
						*ushortImagePtr++ = *ushortColorPtr;
						*ushortImagePtr++ = 0xffff;
					}
					ushortImagePtr += _extraRowWidth;
					oldUShortImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[*oldUShortImagePtr++ * 3];

						*ushortImagePtr++ = *intColorPtr++ / 32768;
						*ushortImagePtr++ = *intColorPtr++ / 32768;
						*ushortImagePtr++ = *intColorPtr / 32768;
						*ushortImagePtr++ = 0xffff;
					}
					ushortImagePtr += _extraRowWidth;
					oldUShortImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[*oldUShortImagePtr++ * 3];

						*ushortImagePtr++ = *uintColorPtr++ / 65536;
						*ushortImagePtr++ = *uintColorPtr++ / 65536;
						*ushortImagePtr++ = *uintColorPtr / 65536;
						*ushortImagePtr++ = 0xffff;
					}
					ushortImagePtr += _extraRowWidth;
					oldUShortImagePtr += oldExtraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[*oldUShortImagePtr++ * 3];

						*ushortImagePtr++ = (unsigned short)(*floatColorPtr++ * 65536);
						*ushortImagePtr++ = (unsigned short)(*floatColorPtr++ * 65536);
						*ushortImagePtr++ = (unsigned short)(*floatColorPtr * 65536);
						*ushortImagePtr++ = (unsigned short)0xffff;
					}
					ushortImagePtr += _extraRowWidth;
					oldUShortImagePtr += oldExtraRowWidth;
				}
			}
		}
		else {      // _imageType == GL_RGB
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ushortImagePtr++ = *oldUShortImagePtr++;
					*ushortImagePtr++ = *oldUShortImagePtr++;
					*ushortImagePtr++ = *oldUShortImagePtr++;
					*ushortImagePtr++ = 0xffff;
				}
				ushortImagePtr += _extraRowWidth;
				oldUShortImagePtr += oldExtraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_INT) {
		GLint *intImagePtr = (GLint*)_imageData;
		GLint *oldIntImagePtr = (GLint*)oldData;
	
		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[*oldIntImagePtr++ * 3];

						*intImagePtr++ = *byteColorPtr++ * 1677216;
						*intImagePtr++ = *byteColorPtr++ * 1677216;
						*intImagePtr++ = *byteColorPtr * 1677216;
						*intImagePtr++ = 0x7fffffff;
					}
					intImagePtr += _extraRowWidth;
					oldIntImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[*oldIntImagePtr++ * 3];

						*intImagePtr++ = *ubyteColorPtr++ * 8388608;
						*intImagePtr++ = *ubyteColorPtr++ * 8388608;
						*intImagePtr++ = *ubyteColorPtr * 8388608;
						*intImagePtr++ = 0x7fffffff;
					}
					intImagePtr += _extraRowWidth;
					oldIntImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[*oldIntImagePtr++ * 3];

						*intImagePtr++ = *shortColorPtr++ * 131072;
						*intImagePtr++ = *shortColorPtr++ * 131072;
						*intImagePtr++ = *shortColorPtr * 131072;
						*intImagePtr++ = 0x7fffffff;
					}
					intImagePtr += _extraRowWidth;
					oldIntImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[*oldIntImagePtr++ * 3];

						*intImagePtr++ = *ushortColorPtr++ * 65536;
						*intImagePtr++ = *ushortColorPtr++ * 65536;
						*intImagePtr++ = *ushortColorPtr * 65536;
						*intImagePtr++ = 0x7fffffff;
					}
					intImagePtr += _extraRowWidth;
					oldIntImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[*oldIntImagePtr++ * 3];

						*intImagePtr++ = *intColorPtr++;
						*intImagePtr++ = *intColorPtr++;
						*intImagePtr++ = *intColorPtr;
						*intImagePtr++ = 0x7fffffff;
					}
					intImagePtr += _extraRowWidth;
					oldIntImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[*oldIntImagePtr++ * 3];

						*intImagePtr++ = *uintColorPtr++ / 2;
						*intImagePtr++ = *uintColorPtr++ / 2;
						*intImagePtr++ = *uintColorPtr / 2;
						*intImagePtr++ = 0x7fffffff;
					}
					intImagePtr += _extraRowWidth;
					oldIntImagePtr += oldExtraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[*oldIntImagePtr++ * 3];

						*intImagePtr++ = (int)(*floatColorPtr++ * (int)(0x7fffffff));
						*intImagePtr++ = (int)(*floatColorPtr++ * (int)(0x7fffffff));
						*intImagePtr++ = (int)(*floatColorPtr++ * (int)(0x7fffffff));
						*intImagePtr++ = (int)0x7fffffff;
					}
					intImagePtr += _extraRowWidth;
					oldIntImagePtr += oldExtraRowWidth;
				}
			}
		}
		else {      // _imageType == GL_RGB
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*intImagePtr++ = *oldIntImagePtr++;
					*intImagePtr++ = *oldIntImagePtr++;
					*intImagePtr++ = *oldIntImagePtr++;
					*intImagePtr++ = 0x7fffffff;
				}
				intImagePtr += _extraRowWidth;
				oldIntImagePtr += oldExtraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_INT) {
		GLuint *uintImagePtr = (GLuint*)_imageData;
		GLuint *oldUIntImagePtr = (GLuint*)oldData;
	
		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[*oldUIntImagePtr++ * 3];

						*uintImagePtr++ = *byteColorPtr++ * (unsigned int)0x01ffffff;
						*uintImagePtr++ = *byteColorPtr++ * (unsigned int)0x01ffffff;
						*uintImagePtr++ = *byteColorPtr * (unsigned int)0x01ffffff;
						*uintImagePtr++ = 0xffffffff;
					}
					uintImagePtr += _extraRowWidth;
					oldUIntImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[*oldUIntImagePtr++ * 3];

						*uintImagePtr++ = *ubyteColorPtr++ * (unsigned int)0x00fffff;
						*uintImagePtr++ = *ubyteColorPtr++ * (unsigned int)0x00fffff;
						*uintImagePtr++ = *ubyteColorPtr * (unsigned int)0x00fffff;
						*uintImagePtr++ = 0xffffffff;
					}
					uintImagePtr += _extraRowWidth;
					oldUIntImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[*oldUIntImagePtr++ * 3];

						*uintImagePtr++ = *shortColorPtr++ * (unsigned int)0x0001ffff;
						*uintImagePtr++ = *shortColorPtr++ * (unsigned int)0x0001ffff;
						*uintImagePtr++ = *shortColorPtr * (unsigned int)0x0001ffff;
						*uintImagePtr++ = 0xffffffff;
					}
					uintImagePtr += _extraRowWidth;
					oldUIntImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[*oldUIntImagePtr++ * 3];

						*uintImagePtr++ = *ushortColorPtr++ * (unsigned int)0x0000ffff;
						*uintImagePtr++ = *ushortColorPtr++ * (unsigned int)0x0000ffff;
						*uintImagePtr++ = *ushortColorPtr * (unsigned int)0x0000ffff;
						*uintImagePtr++ = 0xffffffff;
					}
					uintImagePtr += _extraRowWidth;
					oldUIntImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[*oldUIntImagePtr++ * 3];

						*uintImagePtr++ = *intColorPtr++ * 2;
						*uintImagePtr++ = *intColorPtr++ * 2;
						*uintImagePtr++ = *intColorPtr * 2;
						*uintImagePtr++ = 0xffffffff;
					}
					uintImagePtr += _extraRowWidth;
					oldUIntImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[*oldUIntImagePtr++ * 3];

						*uintImagePtr++ = *uintColorPtr++;
						*uintImagePtr++ = *uintColorPtr++;
						*uintImagePtr++ = *uintColorPtr;
						*uintImagePtr++ = 0xffffffff;
					}
					uintImagePtr += _extraRowWidth;
					oldUIntImagePtr += oldExtraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[*oldUIntImagePtr++ * 3];

						*uintImagePtr++ = (unsigned int)(*floatColorPtr++ * (unsigned int)0xffffffff);
						*uintImagePtr++ = (unsigned int)(*floatColorPtr++ * (unsigned int)0xffffffff);
						*uintImagePtr++ = (unsigned int)(*floatColorPtr * (unsigned int)0xffffffff);
						*uintImagePtr++ = (unsigned int)0xffffffff;
					}
					uintImagePtr += _extraRowWidth;
					oldUIntImagePtr += oldExtraRowWidth;
				}
			}
		}
		else {      // _imageType == GL_RGB
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*uintImagePtr++ = *oldUIntImagePtr++;
					*uintImagePtr++ = *oldUIntImagePtr++;
					*uintImagePtr++ = *oldUIntImagePtr++;
					*uintImagePtr++ = 0xffffffff;
				}
				uintImagePtr += _extraRowWidth;
				oldUIntImagePtr += oldExtraRowWidth;
			}
		}
	}
	else {          // _imageDataType == GL_FLOAT
		GLfloat *floatImagePtr = (GLfloat*)_imageData;
		GLfloat *oldFloatImagePtr = (GLfloat*)oldData;
	
		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[(int)(*oldFloatImagePtr++) * 3];

						*floatImagePtr++ = *byteColorPtr++ / 127.0f;
						*floatImagePtr++ = *byteColorPtr++ / 127.0f;
						*floatImagePtr++ = *byteColorPtr / 127.0f;
						*floatImagePtr++ = 1.0f;
					}
					floatImagePtr += _extraRowWidth;
					oldFloatImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[(int)(*oldFloatImagePtr++) * 3];

						*floatImagePtr++ = *ubyteColorPtr++ / 255.0f;
						*floatImagePtr++ = *ubyteColorPtr++ / 255.0f;
						*floatImagePtr++ = *ubyteColorPtr / 255.0f;
						*floatImagePtr++ = 1.0f;
					}
					floatImagePtr += _extraRowWidth;
					oldFloatImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[(int)(*oldFloatImagePtr++) * 3];

						*floatImagePtr++ = *shortColorPtr++ / 32767.0f;
						*floatImagePtr++ = *shortColorPtr++ / 32767.0f;
						*floatImagePtr++ = *shortColorPtr / 32767.0f;
						*floatImagePtr++ = 1.0f;
					}
					floatImagePtr += _extraRowWidth;
					oldFloatImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[(int)(*oldFloatImagePtr++) * 3];

						*floatImagePtr++ = *ushortColorPtr++ / 65536.0f;
						*floatImagePtr++ = *ushortColorPtr++ / 65536.0f;
						*floatImagePtr++ = *ushortColorPtr / 65536.0f;
						*floatImagePtr++ = 1.0f;
					}
					floatImagePtr += _extraRowWidth;
					oldFloatImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[(int)(*oldFloatImagePtr++) * 3];

						*floatImagePtr++ = *intColorPtr++ / (float)((int)0x7fffffff);
						*floatImagePtr++ = *intColorPtr++ / (float)((int)0x7fffffff);
						*floatImagePtr++ = *intColorPtr / (float)((int)0x7fffffff);
						*floatImagePtr++ = 1.0f;
					}
					floatImagePtr += _extraRowWidth;
					oldFloatImagePtr += oldExtraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[(int)(*oldFloatImagePtr++) * 3];

						*floatImagePtr++ = *uintColorPtr++ / (float)((int)0xffffffff);
						*floatImagePtr++ = *uintColorPtr++ / (float)((int)0xffffffff);
						*floatImagePtr++ = *uintColorPtr / (float)((int)0xffffffff);
						*floatImagePtr++ = 1.0f;
					}
					floatImagePtr += _extraRowWidth;
					oldFloatImagePtr += oldExtraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[(int)(*oldFloatImagePtr++) * 3];

						*floatImagePtr++ = *floatColorPtr++;
						*floatImagePtr++ = *floatColorPtr++;
						*floatImagePtr++ = *floatColorPtr;
						*floatImagePtr++ = 1.0f;
					}
					floatImagePtr += _extraRowWidth;
					oldFloatImagePtr += oldExtraRowWidth;
				}
			}
		}
		else {      // _imageType == GL_RGB
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*floatImagePtr++ = *oldFloatImagePtr++;
					*floatImagePtr++ = *oldFloatImagePtr++;
					*floatImagePtr++ = *oldFloatImagePtr++;
					*floatImagePtr++ = 1.0f;
				}
				floatImagePtr += _extraRowWidth;
				oldFloatImagePtr += oldExtraRowWidth;
			}
		}
	}



	delete [] oldData;
	delete _colormapData;
	_colormapData = NULL;
	_colormapSize = 0;

	_imageType = GL_RGBA;


	return;
}


