// This is ImageCreateNewImage.C
//
// Copyright 1997, 1998     Mark Manca
//


#include "Image.h"



void Image::createNewImage(int width, int height, GLenum type,
                             GLenum dataType, Color backgroundColor) {
	if (_imageData) {
		delete [] _imageData;
	    _imageData = NULL;
	}
	if (_colormapData) {
		delete [] _colormapData;
		_colormapSize = 0;
		_colormapData = NULL;
	}

	_imageType = type;
	_imageDataType = dataType;

	if (_imageType == GL_COLOR_INDEX) {
		_numChannels = 1;
	}
	else if (_imageType == GL_RGB) {
 		_numChannels = 3;
	}
	else {     // assume GL_RGBA
		_numChannels = 4;
	}

	_width = width;
	_height = height;

	_allocateImageData();



	int i, j;


	//
	// duplicate a *LOT* of code to make operations go faster. Also, the large
	// amount of code duplication is necessary due to the different data types
	// that an image can be.
	//

	if (_imageDataType == GL_BYTE) {
		GLbyte *byteImagePtr = (GLbyte*)_imageData;

		GLbyte byteBgColor0 = backgroundColor[0] / 2;
		GLbyte byteBgColor1 = backgroundColor[1] / 2;
		GLbyte byteBgColor2 = backgroundColor[2] / 2;
		GLbyte byteBgColor3 = backgroundColor[3] / 2;

		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*byteImagePtr++ = 0;
				}
				byteImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*byteImagePtr++ = byteBgColor0;
					*byteImagePtr++ = byteBgColor1;
					*byteImagePtr++ = byteBgColor2;
				}
				byteImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*byteImagePtr++ = byteBgColor0;
					*byteImagePtr++ = byteBgColor1;
					*byteImagePtr++ = byteBgColor2;
					*byteImagePtr++ = byteBgColor3;
				}
				byteImagePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_BYTE) {
		GLubyte *ubyteImagePtr = (GLubyte*)_imageData;

		GLubyte ubyteBgColor0 = backgroundColor[0];
		GLubyte ubyteBgColor1 = backgroundColor[1];
		GLubyte ubyteBgColor2 = backgroundColor[2];
		GLubyte ubyteBgColor3 = backgroundColor[3];
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ubyteImagePtr++ = 0;
				}
				ubyteImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ubyteImagePtr++ = ubyteBgColor0;
					*ubyteImagePtr++ = ubyteBgColor1;
					*ubyteImagePtr++ = ubyteBgColor2;
				}
				ubyteImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ubyteImagePtr++ = ubyteBgColor0;
					*ubyteImagePtr++ = ubyteBgColor1;
					*ubyteImagePtr++ = ubyteBgColor2;
					*ubyteImagePtr++ = ubyteBgColor3;
				}
				ubyteImagePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_SHORT) {
		GLshort *shortImagePtr = (GLshort*)_imageData;

		GLshort shortBgColor0 = backgroundColor[0] * 128;
		GLshort shortBgColor1 = backgroundColor[1] * 128;
		GLshort shortBgColor2 = backgroundColor[2] * 128;
		GLshort shortBgColor3 = backgroundColor[3] * 128;
		
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*shortImagePtr++ = 0;
				}
				shortImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*shortImagePtr++ = shortBgColor0;
					*shortImagePtr++ = shortBgColor1;
					*shortImagePtr++ = shortBgColor2;
				}
				shortImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*shortImagePtr++ = shortBgColor0;
					*shortImagePtr++ = shortBgColor1;
					*shortImagePtr++ = shortBgColor2;
					*shortImagePtr++ = shortBgColor3;
				}
				shortImagePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_SHORT) {
		GLushort *ushortImagePtr = (GLushort*)_imageData;

		GLushort ushortBgColor0 = backgroundColor[0] * 256;
		GLushort ushortBgColor1 = backgroundColor[1] * 256;
		GLushort ushortBgColor2 = backgroundColor[2] * 256;
		GLushort ushortBgColor3 = backgroundColor[3] * 256;

		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ushortImagePtr++ = 0;
				}
				ushortImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ushortImagePtr++ = ushortBgColor0;
					*ushortImagePtr++ = ushortBgColor1;
					*ushortImagePtr++ = ushortBgColor2;
				}
				ushortImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ushortImagePtr++ = ushortBgColor0;
					*ushortImagePtr++ = ushortBgColor1;
					*ushortImagePtr++ = ushortBgColor2;
					*ushortImagePtr++ = ushortBgColor3;
				}
				ushortImagePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_INT) {
		GLint *intImagePtr = (GLint*)_imageData;

		GLint intBgColor0 = backgroundColor[0] * 8388608;
		GLint intBgColor1 = backgroundColor[1] * 8388608;  
		GLint intBgColor2 = backgroundColor[2] * 8388608;  
		GLint intBgColor3 = backgroundColor[3] * 8388608;  
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*intImagePtr++ = 0;
				}
				intImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*intImagePtr++ = intBgColor0;
					*intImagePtr++ = intBgColor1;
					*intImagePtr++ = intBgColor2;
				}
				intImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*intImagePtr++ = intBgColor0;
					*intImagePtr++ = intBgColor1;
					*intImagePtr++ = intBgColor2;
					*intImagePtr++ = intBgColor3;
				}
				intImagePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_INT) {
		GLuint *uintImagePtr = (GLuint*)_imageData;

		GLuint uintBgColor0 = backgroundColor[0] * 16777216;
		GLuint uintBgColor1 = backgroundColor[1] * 16777216;
		GLuint uintBgColor2 = backgroundColor[2] * 16777216;
		GLuint uintBgColor3 = backgroundColor[3] * 16777216;
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*uintImagePtr++ = 0;
				}
				uintImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*uintImagePtr++ = uintBgColor0;
					*uintImagePtr++ = uintBgColor1;
					*uintImagePtr++ = uintBgColor2;
				}
				uintImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*uintImagePtr++ = uintBgColor0;
					*uintImagePtr++ = uintBgColor1;
					*uintImagePtr++ = uintBgColor2;
					*uintImagePtr++ = uintBgColor3;
				}
				uintImagePtr += _extraRowWidth;
			}
		}
	}
	else {          // _imageDataType == GL_FLOAT
		GLfloat *floatImagePtr = (GLfloat*)_imageData;

		GLfloat floatBgColor0 = (float)backgroundColor[0] / 255.0f;
		GLfloat floatBgColor1 = (float)backgroundColor[1] / 255.0f;
		GLfloat floatBgColor2 = (float)backgroundColor[2] / 255.0f;
		GLfloat floatBgColor3 = (float)backgroundColor[3] / 255.0f;
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*floatImagePtr++ = 0;
				}
				floatImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*floatImagePtr++ = floatBgColor0;
					*floatImagePtr++ = floatBgColor1;
					*floatImagePtr++ = floatBgColor2;
				}
				floatImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*floatImagePtr++ = floatBgColor0;
					*floatImagePtr++ = floatBgColor1;
					*floatImagePtr++ = floatBgColor2;
					*floatImagePtr++ = floatBgColor3;
				}
				floatImagePtr += _extraRowWidth;
			}
		}
	}
}




