// This is ImageFile.h
//
// Copyright 1997, 1998     Mark Manca
//



#ifndef __IMAGEFILE_H
#define __IMAGEFILE_H

#include <stdio.h>

#include "Image.h"

struct rgbFileHeader;
struct bmpFileHeader;
struct bmpInfoHeader;
struct gifFileHeader;


class ImageFile : public Image {
	public:
		//
		// different file types. Right now only RGB and GIF87a are supported
		//
		enum ImageFileType {
			UNKNOWN_FILE, WVLT_FILE, RGB_FILE, GIF87A_FILE, GIF89A_FILE,
			BMP_FILE, PIX_FILE, PNG_FILE, PPM_FILE, TIFF_FILE, JPG_FILE, 
			IMAGEFILE_ERROR 
		};

		//
		// constructors, destructors, etc...
		//
		ImageFile(const char *filename=NULL, int read=1, int open=1);
		~ImageFile();


		int setFilename(const char *filename);
		const char *getFilename();

		void drawWhileRead(int state);


		int open(const char *type="rb");
		int open(const char *filename, const char *type);

		int readDimensions();
		int readDimensions(const char *filename);

		//
		// these read functions should only be called after a 
		// gl context has been created and made current
		//
		int read();
		int read(const char *filename);
		
		int write(ImageFileType type=RGB_FILE);
		int write(const char *filename, ImageFileType type=RGB_FILE);

		int close();


		ImageFile& operator=(const Image& image);
		ImageFile& operator=(const ImageFile& image);

		
	protected:
		ImageFileType _findImageFileType();

		//
		// these should only be called after a gl context 
		// has been made current
		//
		int _readRGBFile();
		int _readBMPFile();
		int _readGIF87aFile();


		//
		// these functions can be called at any time.
		//
		int _readRGBHeader();
		int _readRGBHeader(rgbFileHeader* header);
		int _readBMPHeader();
		int _readBMPHeader(bmpFileHeader* fileHeader, bmpInfoHeader* infoHeader);
		int _readGIF87aHeader();
		int _readGIF87aHeader(gifFileHeader* header);


		//
		// functions to write out a particular file format
		//
		int _writeRGBFile();
		int _writeBMPFile();
		int _writeGIF87aFile();


		ImageFileType _imageFileType;


	private:
		FILE* __imageFile;
		char* __filename;
		int __isOpen;

		int __drawWhileRead;
};

#endif		// __IMAGEFILE_H
