// This is ImageFileOperatorEqual.C
//
// Copyright 1997, 1998     Mark Manca
//

#include <string.h>
#include "ImageFile.h"



ImageFile& ImageFile::operator=(const Image& image) {
	Image::operator=(image);

	return *this;
}




ImageFile& ImageFile::operator=(const ImageFile& imageFile) {
	if (_imageData) {
		delete [] _imageData;
		_imageData = NULL;
	}
	if (_colormapData) {
		delete [] _colormapData;
		_colormapData = NULL;
		_colormapSize = 0;
	}
	if (_imageName) {
		delete [] _imageName;
		_imageName = NULL;
	}

	__isOpen = 0;
	__drawWhileRead = imageFile.__drawWhileRead;

	_width = imageFile._width;
	_height = imageFile._height;
	_xScale = imageFile._xScale;
	_yScale = imageFile._yScale;

	if (imageFile._imageName) {
		#ifdef WIN32
			_imageName = _strdup(imageFile._imageName);
		#else
			_imageName = strdup(imageFile._imageName);
		#endif
	}
	else {
		_imageName = NULL;
	}

	if (imageFile.__filename) {
		#ifdef WIN32
			__filename = _strdup(imageFile.__filename);
		#else
			__filename = strdup(imageFile.__filename);
		#endif
	}
	else {
		__filename = NULL;
	}


	_imageType = imageFile._imageType;
	_imageDataType = imageFile._imageDataType;
	_numChannels = imageFile._numChannels;
	_numChannels = imageFile._numChannels;
	_extraRowWidth = imageFile._extraRowWidth;

	_allocateImageData();




	//
	// now begin the actual coping of the data
	//

	int i, j;


	//
	// duplicate a *LOT* of code to make operations go faster. Also, the large
	// amount of code duplication is necessary due to the different data types
	// that an image can be.
	//

	if (_imageDataType == GL_BYTE) {
		GLbyte *byteImageFilePtr = (GLbyte*)_imageData;
		GLbyte *otherByteImageFilePtr = (GLbyte*)imageFile._imageData;

		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*byteImageFilePtr++ = *otherByteImageFilePtr++;
				}
				byteImageFilePtr += _extraRowWidth;
				otherByteImageFilePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*byteImageFilePtr++ = *otherByteImageFilePtr++;
					*byteImageFilePtr++ = *otherByteImageFilePtr++;
					*byteImageFilePtr++ = *otherByteImageFilePtr++;
				}
				byteImageFilePtr += _extraRowWidth;
				otherByteImageFilePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*byteImageFilePtr++ = *otherByteImageFilePtr++;
					*byteImageFilePtr++ = *otherByteImageFilePtr++;
					*byteImageFilePtr++ = *otherByteImageFilePtr++;
					*byteImageFilePtr++ = *otherByteImageFilePtr++;
				}
				byteImageFilePtr += _extraRowWidth;
				otherByteImageFilePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_BYTE) {
		GLubyte *ubyteImageFilePtr = (GLubyte*)_imageData;
		GLubyte *otherUByteImageFilePtr = (GLubyte*)imageFile._imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ubyteImageFilePtr++ = *otherUByteImageFilePtr++;
				}
				ubyteImageFilePtr += _extraRowWidth;
				otherUByteImageFilePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ubyteImageFilePtr++ = *otherUByteImageFilePtr++;
					*ubyteImageFilePtr++ = *otherUByteImageFilePtr++;
					*ubyteImageFilePtr++ = *otherUByteImageFilePtr++;
				}
				ubyteImageFilePtr += _extraRowWidth;
				otherUByteImageFilePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ubyteImageFilePtr++ = *otherUByteImageFilePtr++;
					*ubyteImageFilePtr++ = *otherUByteImageFilePtr++;
					*ubyteImageFilePtr++ = *otherUByteImageFilePtr++;
					*ubyteImageFilePtr++ = *otherUByteImageFilePtr++;
				}
				ubyteImageFilePtr += _extraRowWidth;
				otherUByteImageFilePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_SHORT) {
		GLshort *shortImageFilePtr = (GLshort*)_imageData;
		GLshort *otherShortImageFilePtr = (GLshort*)imageFile._imageData;
		
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*shortImageFilePtr++ = *otherShortImageFilePtr++;
				}
				shortImageFilePtr += _extraRowWidth;
				otherShortImageFilePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*shortImageFilePtr++ = *otherShortImageFilePtr++;
					*shortImageFilePtr++ = *otherShortImageFilePtr++;
					*shortImageFilePtr++ = *otherShortImageFilePtr++;
				}
				shortImageFilePtr += _extraRowWidth;
				otherShortImageFilePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*shortImageFilePtr++ = *otherShortImageFilePtr++;
					*shortImageFilePtr++ = *otherShortImageFilePtr++;
					*shortImageFilePtr++ = *otherShortImageFilePtr++;
					*shortImageFilePtr++ = *otherShortImageFilePtr++;
				}
				shortImageFilePtr += _extraRowWidth;
				otherShortImageFilePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_SHORT) {
		GLushort *ushortImageFilePtr = (GLushort*)_imageData;
		GLushort *otherUShortImageFilePtr = (GLushort*)imageFile._imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ushortImageFilePtr++ = *otherUShortImageFilePtr++;
				}
				ushortImageFilePtr += _extraRowWidth;
				otherUShortImageFilePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ushortImageFilePtr++ = *otherUShortImageFilePtr++;
					*ushortImageFilePtr++ = *otherUShortImageFilePtr++;
					*ushortImageFilePtr++ = *otherUShortImageFilePtr++;
				}
				ushortImageFilePtr += _extraRowWidth;
				otherUShortImageFilePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ushortImageFilePtr++ = *otherUShortImageFilePtr++;
					*ushortImageFilePtr++ = *otherUShortImageFilePtr++;
					*ushortImageFilePtr++ = *otherUShortImageFilePtr++;
					*ushortImageFilePtr++ = *otherUShortImageFilePtr++;
				}
				ushortImageFilePtr += _extraRowWidth;
				otherUShortImageFilePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_INT) {
		GLint *intImageFilePtr = (GLint*)_imageData;
		GLint *otherIntImageFilePtr = (GLint*)imageFile._imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*intImageFilePtr++ = *otherIntImageFilePtr++;
				}
				intImageFilePtr += _extraRowWidth;
				otherIntImageFilePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*intImageFilePtr++ = *otherIntImageFilePtr++;
					*intImageFilePtr++ = *otherIntImageFilePtr++;
					*intImageFilePtr++ = *otherIntImageFilePtr++;
				}
				intImageFilePtr += _extraRowWidth;
				otherIntImageFilePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*intImageFilePtr++ = *otherIntImageFilePtr++;
					*intImageFilePtr++ = *otherIntImageFilePtr++;
					*intImageFilePtr++ = *otherIntImageFilePtr++;
					*intImageFilePtr++ = *otherIntImageFilePtr++;
				}
				intImageFilePtr += _extraRowWidth;
				otherIntImageFilePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_INT) {
		GLuint *uintImageFilePtr = (GLuint*)_imageData;
		GLuint *otherUIntImageFilePtr = (GLuint*)imageFile._imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*uintImageFilePtr++ = *otherUIntImageFilePtr++;
				}
				uintImageFilePtr += _extraRowWidth;
				otherUIntImageFilePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*uintImageFilePtr++ = *otherUIntImageFilePtr++;
					*uintImageFilePtr++ = *otherUIntImageFilePtr++;
					*uintImageFilePtr++ = *otherUIntImageFilePtr++;
				}
				uintImageFilePtr += _extraRowWidth;
				otherUIntImageFilePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*uintImageFilePtr++ = *otherUIntImageFilePtr++;
					*uintImageFilePtr++ = *otherUIntImageFilePtr++;
					*uintImageFilePtr++ = *otherUIntImageFilePtr++;
					*uintImageFilePtr++ = *otherUIntImageFilePtr++;
				}
				uintImageFilePtr += _extraRowWidth;
				otherUIntImageFilePtr += _extraRowWidth;
			}
		}
	}
	else {          // _imageDataType == GL_FLOAT
		GLfloat *floatImageFilePtr = (GLfloat*)_imageData;
		GLfloat *otherFloatImageFilePtr = (GLfloat*)imageFile._imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*floatImageFilePtr++ = *otherFloatImageFilePtr++;
				}
				floatImageFilePtr += _extraRowWidth;
				otherFloatImageFilePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*floatImageFilePtr++ = *otherFloatImageFilePtr++;
					*floatImageFilePtr++ = *otherFloatImageFilePtr++;
					*floatImageFilePtr++ = *otherFloatImageFilePtr++;
				}
				floatImageFilePtr += _extraRowWidth;
				otherFloatImageFilePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*floatImageFilePtr++ = *otherFloatImageFilePtr++;
					*floatImageFilePtr++ = *otherFloatImageFilePtr++;
					*floatImageFilePtr++ = *otherFloatImageFilePtr++;
					*floatImageFilePtr++ = *otherFloatImageFilePtr++;
				}
				floatImageFilePtr += _extraRowWidth;
				otherFloatImageFilePtr += _extraRowWidth;
			}
		}
	}




	//
	// copy the colormap information
	//
	_colormapSize = imageFile._colormapSize;
	_colormapDataType = imageFile._colormapDataType;
	_allocateColormapData();


	//
	// duplicate a *LOT* of code to make operations go faster. Also, the large
	// amount of code duplication is necessary due to the different data types
	// that an image can be.
	//

	if (_colormapDataType == GL_BYTE) {
		GLbyte *byteColormapPtr = (GLbyte*)_colormapData;
		GLbyte *otherByteColormapPtr = (GLbyte*)imageFile._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*byteColormapPtr++ = *otherByteColormapPtr++;
			*byteColormapPtr++ = *otherByteColormapPtr++;
			*byteColormapPtr++ = *otherByteColormapPtr++;
		}
	}
	else if (_colormapDataType == GL_UNSIGNED_BYTE) {
		GLubyte *ubyteColormapPtr = (GLubyte*)_colormapData;
		GLubyte *otherUByteColormapPtr = (GLubyte*)imageFile._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*ubyteColormapPtr++ = *otherUByteColormapPtr++;
			*ubyteColormapPtr++ = *otherUByteColormapPtr++;
			*ubyteColormapPtr++ = *otherUByteColormapPtr++;
		}
	}
	else if (_colormapDataType == GL_SHORT) {
		GLshort *shortColormapPtr = (GLshort*)_colormapData;
		GLshort *otherShortColormapPtr = (GLshort*)imageFile._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*shortColormapPtr++ = *otherShortColormapPtr++;
			*shortColormapPtr++ = *otherShortColormapPtr++;
			*shortColormapPtr++ = *otherShortColormapPtr++;
		}
	}
	else if (_colormapDataType == GL_UNSIGNED_SHORT) {
		GLushort *ushortColormapPtr = (GLushort*)_colormapData;
		GLushort *otherUShortColormapPtr = (GLushort*)imageFile._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*ushortColormapPtr++ = *otherUShortColormapPtr++;
			*ushortColormapPtr++ = *otherUShortColormapPtr++;
			*ushortColormapPtr++ = *otherUShortColormapPtr++;
		}
	}
	else if (_colormapDataType == GL_INT) {
		GLint *intColormapPtr = (GLint*)_colormapData;
		GLint *otherIntColormapPtr = (GLint*)imageFile._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*intColormapPtr++ = *otherIntColormapPtr++;
			*intColormapPtr++ = *otherIntColormapPtr++;
			*intColormapPtr++ = *otherIntColormapPtr++;
		}
	}
	else if (_colormapDataType == GL_UNSIGNED_INT) {
		GLuint *uintColormapPtr = (GLuint*)_colormapData;
		GLuint *otherUIntColormapPtr = (GLuint*)imageFile._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*uintColormapPtr++ = *otherUIntColormapPtr++;
			*uintColormapPtr++ = *otherUIntColormapPtr++;
			*uintColormapPtr++ = *otherUIntColormapPtr++;
		}
	}
	else {          // _colormapDataType == GL_FLOAT
		GLfloat *floatColormapPtr = (GLfloat*)_colormapData;
		GLfloat *otherFloatColormapPtr = (GLfloat*)imageFile._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*floatColormapPtr++ = *otherFloatColormapPtr++;
			*floatColormapPtr++ = *otherFloatColormapPtr++;
			*floatColormapPtr++ = *otherFloatColormapPtr++;
		}
	}



	return *this;
}




