// This is ImageGetIntensity.C
//
// Copyright 1997, 1998     Mark Manca
//

#include <string.h>
#include "Image.h"


Image* Image::getIntensity() {
	Image* image = new Image;
	int i, j;


	image->_width = _width;
	image->_height = _height;
	image->_xScale = _xScale;
	image->_yScale = _yScale;

	if (_imageName) {
		#ifdef WIN32
			image->_imageName = _strdup(_imageName);
		#else
			image->_imageName = strdup(_imageName);
		#endif
	}
	else {
		image->_imageName = NULL;
	}

    image->_imageType = GL_COLOR_INDEX;
    image->_numChannels = 1;



	//
	// create a color map for the new image that is totally made up of
	// black and white values
	//
	image->_imageDataType = GL_UNSIGNED_BYTE;
	image->_colormapSize = 256;
	image->_colormapDataType = GL_UNSIGNED_BYTE;

	image->_allocateImageData();
	image->_allocateColormapData();


	GLubyte *newColormapPtr = (GLubyte*)(image->_colormapData);
	GLubyte *newImageData = (GLubyte*)(image->_imageData);

	for (i=0; i < image->_colormapSize; i++) {
		*newColormapPtr++ = (GLubyte)i;
		*newColormapPtr++ = (GLubyte)i;
		*newColormapPtr++ = (GLubyte)i;
	}




	//
	// duplicate a *LOT* of code to make operations go faster. Also, the large
	// amount of code duplication is necessary due to the different data types
	// that an image can be.
	//
	GLbyte *byteColormapPtr = (GLbyte*)_colormapData;
	GLubyte *ubyteColormapPtr = (GLubyte*)_colormapData;
	GLshort *shortColormapPtr = (GLshort*)_colormapData;
	GLushort *ushortColormapPtr = (GLushort*)_colormapData;
	GLint *intColormapPtr = (GLint*)_colormapData;
	GLuint *uintColormapPtr = (GLuint*)_colormapData;
	GLfloat *floatColormapPtr = (GLfloat*)_colormapData;
    
	GLbyte *byteColorPtr;
	GLubyte *ubyteColorPtr;
	GLshort *shortColorPtr;
	GLushort *ushortColorPtr;
	GLint *intColorPtr;
	GLuint *uintColorPtr;
	GLfloat *floatColorPtr;


	if (_imageDataType == GL_BYTE) {
		GLbyte *byteImagePtr = (GLbyte*)_imageData;

		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[*byteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *byteColorPtr++ +
						                             59 * *byteColorPtr++ +
						                             11 * *byteColorPtr++)
						                            / 50);
					}
					byteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[*byteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *ubyteColorPtr++ +
						                             59 * *ubyteColorPtr++ +
						                             11 * *ubyteColorPtr++)
						                            / 100);
					}
					byteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[*byteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *shortColorPtr++ +
						                             59 * *shortColorPtr++ +
						                             11 * *shortColorPtr++)
						                            / 12800);
					}
					byteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[*byteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *ushortColorPtr++ +
						                             59 * *ushortColorPtr++ +
						                             11 * *ushortColorPtr++)
						                            / 25600);
					}
					byteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[*byteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *intColorPtr++ +
						                             59 * *intColorPtr++ +
						                             11 * *intColorPtr++)
						                            / ((int)0x007fffff * 100));
					}
					byteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[*byteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *uintColorPtr++ +
						                             59 * *uintColorPtr++ +
						                             11 * *uintColorPtr++)
						                            / ((int)0x00ffffff * 100));
					}
					byteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[*byteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30.0f * *floatColorPtr++ +
						                             59.0f * *floatColorPtr++ +
						                             11.0f * *floatColorPtr++)
						                            * 2.55f);
					}
					byteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30 * *byteColorPtr++ +
					                             59 * *byteColorPtr++ +
					                             11 * *byteColorPtr++)
					                            / 50);
				}
				byteImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30 * *byteColorPtr++ +
					                             59 * *byteColorPtr++ +
					                             11 * *byteColorPtr++)
					                            / 50);
					byteImagePtr++;
				}
				byteImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_BYTE) {
		GLubyte *ubyteImagePtr = (GLubyte*)_imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[*ubyteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *byteColorPtr++ +
						                             59 * *byteColorPtr++ +
						                             11 * *byteColorPtr++)
						                            / 50);
					}
					ubyteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[*ubyteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *ubyteColorPtr++ +
						                             59 * *ubyteColorPtr++ +
						                             11 * *ubyteColorPtr++)
						                            / 100);
					}
					ubyteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[*ubyteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *shortColorPtr++ +
						                             59 * *shortColorPtr++ +
						                             11 * *shortColorPtr++)
						                            / 12800);
					}
					ubyteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[*ubyteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *ushortColorPtr++ +
						                             59 * *ushortColorPtr++ +
						                             11 * *ushortColorPtr++)
						                            / 25600);
					}
					ubyteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[*ubyteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *intColorPtr++ +
						                             59 * *intColorPtr++ +
						                             11 * *intColorPtr++)
						                            / ((int)0x007fffff * 100));
					}
					ubyteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[*ubyteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *uintColorPtr++ +
						                             59 * *uintColorPtr++ +
						                             11 * *uintColorPtr++)
						                            / ((int)0x00ffffff * 100));
					}
					ubyteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[*ubyteImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30.0f * *floatColorPtr++ +
						                             59.0f * *floatColorPtr++ +
						                             11.0f * *floatColorPtr++)
						                            * 2.55f);
					}
					ubyteImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30 * *ubyteImagePtr++ +
					                             59 * *ubyteImagePtr++ +
					                             11 * *ubyteImagePtr++)
					                            / 100);
				}
				ubyteImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30 * *ubyteImagePtr++ +
					                             59 * *ubyteImagePtr++ +
					                             11 * *ubyteImagePtr++)
					                            / 100);
					ubyteImagePtr++;
				}
				ubyteImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_SHORT) {
		GLshort *shortImagePtr = (GLshort*)_imageData;
		
		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[*shortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *byteColorPtr++ +
						                             59 * *byteColorPtr++ +
						                             11 * *byteColorPtr++)
						                            / 50);
					}
					shortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[*shortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *ubyteColorPtr++ +
						                             59 * *ubyteColorPtr++ +
						                             11 * *ubyteColorPtr++)
						                            / 100);
					}
					shortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[*shortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *shortColorPtr++ +
						                             59 * *shortColorPtr++ +
						                             11 * *shortColorPtr++)
						                            / 12800);
					}
					shortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[*shortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *ushortColorPtr++ +
						                             59 * *ushortColorPtr++ +
						                             11 * *ushortColorPtr++)
						                            / 25600);
					}
					shortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[*shortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *intColorPtr++ +
						                             59 * *intColorPtr++ +
						                             11 * *intColorPtr++)
						                            / ((int)0x007fffff * 100));
					}
					shortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[*shortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *uintColorPtr++ +
						                             59 * *uintColorPtr++ +
						                             11 * *uintColorPtr++)
						                            / ((int)0x00ffffff * 100));
					}
					shortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[*shortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30.0f * *floatColorPtr++ +
						                             59.0f * *floatColorPtr++ +
						                             11.0f * *floatColorPtr++)
						                            * 2.55f);
					}
					shortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30 * *shortImagePtr++ +
					                             59 * *shortImagePtr++ +
					                             11 * *shortImagePtr++)
					                            / 12800);
				}
				shortImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30 * *shortImagePtr++ +
					                             59 * *shortImagePtr++ +
					                             11 * *shortImagePtr++)
					                            / 12800);
					shortImagePtr++;
				}
				shortImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_SHORT) {
		GLushort *ushortImagePtr = (GLushort*)_imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[*ushortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *byteColorPtr++ +
						                             59 * *byteColorPtr++ +
						                             11 * *byteColorPtr++)
						                            / 50);
					}
					ushortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[*ushortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *ubyteColorPtr++ +
						                             59 * *ubyteColorPtr++ +
						                             11 * *ubyteColorPtr++)
						                            / 100);
					}
					ushortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[*ushortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *shortColorPtr++ +
						                             59 * *shortColorPtr++ +
						                             11 * *shortColorPtr++)
						                            / 12800);
					}
					ushortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[*ushortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *ushortColorPtr++ +
						                             59 * *ushortColorPtr++ +
						                             11 * *ushortColorPtr++)
						                            / 25600);
					}
					ushortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[*ushortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *intColorPtr++ +
						                             59 * *intColorPtr++ +
						                             11 * *intColorPtr++)
						                            / ((int)0x007fffff * 100));
					}
					ushortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[*ushortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *uintColorPtr++ +
						                             59 * *uintColorPtr++ +
						                             11 * *uintColorPtr++)
						                            / ((int)0x00ffffff * 100));
					}
					ushortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[*ushortImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30.0f * *floatColorPtr++ +
						                             59.0f * *floatColorPtr++ +
						                             11.0f * *floatColorPtr++)
						                            * 2.55f);
					}
					ushortImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30 * *ushortImagePtr++ +
					                             59 * *ushortImagePtr++ +
					                             11 * *ushortImagePtr++)
					                            / 25600);
				}
				ushortImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30 * *ushortImagePtr++ +
					                             59 * *ushortImagePtr++ +
					                             11 * *ushortImagePtr++)
					                            / 25600);
					ushortImagePtr++;
				}
				ushortImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_INT) {
		GLint *intImagePtr = (GLint*)_imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[*intImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *byteColorPtr++ +
						                             59 * *byteColorPtr++ +
						                             11 * *byteColorPtr++)
						                            / 50);
					}
					intImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[*intImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *ubyteColorPtr++ +
						                             59 * *ubyteColorPtr++ +
						                             11 * *ubyteColorPtr++)
						                            / 100);
					}
					intImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[*intImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *shortColorPtr++ +
						                             59 * *shortColorPtr++ +
						                             11 * *shortColorPtr++)
						                            / 12800);
					}
					intImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[*intImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *ushortColorPtr++ +
						                             59 * *ushortColorPtr++ +
						                             11 * *ushortColorPtr++)
						                            / 25600);
					}
					intImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[*intImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *intColorPtr++ +
						                             59 * *intColorPtr++ +
						                             11 * *intColorPtr++)
						                            / ((int)0x007fffff * 100));
					}
					intImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[*intImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *uintColorPtr++ +
						                             59 * *uintColorPtr++ +
						                             11 * *uintColorPtr++)
						                            / ((int)0x00ffffff * 100.0));
					}
					intImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[*intImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30.0f * *floatColorPtr++ +
						                             59.0f * *floatColorPtr++ +
						                             11.0f * *floatColorPtr++)
						                            * 2.55f);
					}
					intImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30 * *intImagePtr++ +
					                             59 * *intImagePtr++ +
					                             11 * *intImagePtr++)
					                            / ((int)0x007fffff * 100));
				}
				intImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30 * *intImagePtr++ +
					                             59 * *intImagePtr++ +
					                             11 * *intImagePtr++)
					                            / ((int)0x007fffff * 100));
					intImagePtr++;
				}
				intImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_INT) {
		GLuint *uintImagePtr = (GLuint*)_imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[*uintImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *byteColorPtr++ +
						                             59 * *byteColorPtr++ +
						                             11 * *byteColorPtr++)
						                            / 50);
					}
					uintImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[*uintImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *ubyteColorPtr++ +
						                             59 * *ubyteColorPtr++ +
						                             11 * *ubyteColorPtr++)
						                            / 100);
					}
					uintImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[*uintImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *shortColorPtr++ +
						                             59 * *shortColorPtr++ +
						                             11 * *shortColorPtr++)
						                            / 12800);
					}
					uintImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[*uintImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *ushortColorPtr++ +
						                             59 * *ushortColorPtr++ +
						                             11 * *ushortColorPtr++)
						                            / 25600);
					}
					uintImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[*uintImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *intColorPtr++ +
						                             59 * *intColorPtr++ +
						                             11 * *intColorPtr++)
						                            / ((int)0x007fffff * 100));
					}
					uintImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[*uintImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30 * *uintColorPtr++ +
						                             59 * *uintColorPtr++ +
						                             11 * *uintColorPtr++)
						                            / ((int)0x00ffffff * 100.0));
					}
					uintImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[*uintImagePtr++ * 3];

						*newImageData++ = (GLubyte)((30.0f * *floatColorPtr++ +
						                             59.0f * *floatColorPtr++ +
						                             11.0f * *floatColorPtr++)
						                            * 2.55f);
					}
					uintImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30 * *uintImagePtr++ +
					                             59 * *uintImagePtr++ +
					                             11 * *uintImagePtr++)
					                            / ((int)0x00ffffff * 100));
				}
				uintImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30 * *uintImagePtr++ +
					                             59 * *uintImagePtr++ +
					                             11 * *uintImagePtr++)
					                            / ((int)0x00ffffff * 100));
					uintImagePtr++;
				}
				uintImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
	}
	else {          // _imageDataType == GL_FLOAT
		GLfloat *floatImagePtr = (GLfloat*)_imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			if (_colormapDataType == GL_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						byteColorPtr = &byteColormapPtr[(int)(*floatImagePtr++) * 3];

						*newImageData++ = (GLubyte)((30 * *byteColorPtr++ +
						                             59 * *byteColorPtr++ +
						                             11 * *byteColorPtr++)
						                            / 50);
					}
					floatImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_BYTE) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ubyteColorPtr = &ubyteColormapPtr[(int)(*floatImagePtr++) * 3];

						*newImageData++ = (GLubyte)((30 * *ubyteColorPtr++ +
						                             59 * *ubyteColorPtr++ +
						                             11 * *ubyteColorPtr++)
						                            / 100);
					}
					floatImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						shortColorPtr = &shortColormapPtr[(int)(*floatImagePtr++) * 3];

						*newImageData++ = (GLubyte)((30 * *shortColorPtr++ +
						                             59 * *shortColorPtr++ +
						                             11 * *shortColorPtr++)
						                            / 12800);
					}
					floatImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_SHORT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						ushortColorPtr = &ushortColormapPtr[(int)(*floatImagePtr++) * 3];

						*newImageData++ = (GLubyte)((30 * *ushortColorPtr++ +
						                             59 * *ushortColorPtr++ +
						                             11 * *ushortColorPtr++)
						                            / 25600);
					}
					floatImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						intColorPtr = &intColormapPtr[(int)(*floatImagePtr++) * 3];

						*newImageData++ = (GLubyte)((30 * *intColorPtr++ +
						                             59 * *intColorPtr++ +
						                             11 * *intColorPtr++)
						                            / ((int)0x007fffff * 100));
					}
					floatImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else if (_colormapDataType == GL_UNSIGNED_INT) {
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						uintColorPtr = &uintColormapPtr[(int)(*floatImagePtr++) * 3];

						*newImageData++ = (GLubyte)((30 * *uintColorPtr++ +
						                             59 * *uintColorPtr++ +
						                             11 * *uintColorPtr++)
						                            / ((int)0x00ffffff * 100));
					}
					floatImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
			else {          // _colormapDataType == GL_FLOAT
				for (i=_height; i > 0; i--) {
					for (j=_width; j > 0; j--) {
						floatColorPtr = &floatColormapPtr[(int)(*floatImagePtr++) * 3];

						*newImageData++ = (GLubyte)((30.0f * *floatColorPtr++ +
						                             59.0f * *floatColorPtr++ +
						                             11.0f * *floatColorPtr++)
						                            * 2.55f);
					}
					floatImagePtr += _extraRowWidth;
					newImageData += image->_extraRowWidth;
				}
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30.0f * *floatImagePtr++ +
					                             59.0f * *floatImagePtr++ +
					                             11.0f * *floatImagePtr++)
					                            / 2.55f);
				}
				floatImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*newImageData++ = (GLubyte)((30.0f * *floatImagePtr++ +
					                             59.0f * *floatImagePtr++ +
					                             11.0f * *floatImagePtr++)
					                            / 2.55f);
					floatImagePtr++;
				}
				floatImagePtr += _extraRowWidth;
				newImageData += image->_extraRowWidth;
			}
		}
	}



	return image;
}



