// This is ImageOperatorEqual.C
//
// Copyright 1997, 1998     Mark Manca
//

#include <string.h>
#include "Image.h"



Image& Image::operator=(const Image& image) {
	if (_imageData) {
		delete [] _imageData;
		_imageData = NULL;
	}
	if (_colormapData) {
		delete [] _colormapData;
		_colormapData = NULL;
		_colormapSize = 0;
	}
	if (_imageName) {
		delete [] _imageName;
		_imageName = NULL;
	}

	_width = image._width;
	_height = image._height;
	_xScale = image._xScale;
	_yScale = image._yScale;

	if (image._imageName) {
		#ifdef WIN32
			_imageName = _strdup(image._imageName);
		#else
			_imageName = strdup(image._imageName);
		#endif
	}
	else {
		_imageName = NULL;
	}

	_imageType = image._imageType;
	_imageDataType = image._imageDataType;
	_numChannels = image._numChannels;
	_numChannels = image._numChannels;
	_extraRowWidth = image._extraRowWidth;

	_allocateImageData();




	//
	// now begin the actual coping of the data
	//

	int i, j;


	//
	// duplicate a *LOT* of code to make operations go faster. Also, the large
	// amount of code duplication is necessary due to the different data types
	// that an image can be.
	//

	if (_imageDataType == GL_BYTE) {
		GLbyte *byteImagePtr = (GLbyte*)_imageData;
		GLbyte *otherByteImagePtr = (GLbyte*)image._imageData;

		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*byteImagePtr++ = *otherByteImagePtr++;
				}
				byteImagePtr += _extraRowWidth;
				otherByteImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*byteImagePtr++ = *otherByteImagePtr++;
					*byteImagePtr++ = *otherByteImagePtr++;
					*byteImagePtr++ = *otherByteImagePtr++;
				}
				byteImagePtr += _extraRowWidth;
				otherByteImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*byteImagePtr++ = *otherByteImagePtr++;
					*byteImagePtr++ = *otherByteImagePtr++;
					*byteImagePtr++ = *otherByteImagePtr++;
					*byteImagePtr++ = *otherByteImagePtr++;
				}
				byteImagePtr += _extraRowWidth;
				otherByteImagePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_BYTE) {
		GLubyte *ubyteImagePtr = (GLubyte*)_imageData;
		GLubyte *otherUByteImagePtr = (GLubyte*)image._imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ubyteImagePtr++ = *otherUByteImagePtr++;
				}
				ubyteImagePtr += _extraRowWidth;
				otherUByteImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ubyteImagePtr++ = *otherUByteImagePtr++;
					*ubyteImagePtr++ = *otherUByteImagePtr++;
					*ubyteImagePtr++ = *otherUByteImagePtr++;
				}
				ubyteImagePtr += _extraRowWidth;
				otherUByteImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ubyteImagePtr++ = *otherUByteImagePtr++;
					*ubyteImagePtr++ = *otherUByteImagePtr++;
					*ubyteImagePtr++ = *otherUByteImagePtr++;
					*ubyteImagePtr++ = *otherUByteImagePtr++;
				}
				ubyteImagePtr += _extraRowWidth;
				otherUByteImagePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_SHORT) {
		GLshort *shortImagePtr = (GLshort*)_imageData;
		GLshort *otherShortImagePtr = (GLshort*)image._imageData;
		
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*shortImagePtr++ = *otherShortImagePtr++;
				}
				shortImagePtr += _extraRowWidth;
				otherShortImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*shortImagePtr++ = *otherShortImagePtr++;
					*shortImagePtr++ = *otherShortImagePtr++;
					*shortImagePtr++ = *otherShortImagePtr++;
				}
				shortImagePtr += _extraRowWidth;
				otherShortImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*shortImagePtr++ = *otherShortImagePtr++;
					*shortImagePtr++ = *otherShortImagePtr++;
					*shortImagePtr++ = *otherShortImagePtr++;
					*shortImagePtr++ = *otherShortImagePtr++;
				}
				shortImagePtr += _extraRowWidth;
				otherShortImagePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_SHORT) {
		GLushort *ushortImagePtr = (GLushort*)_imageData;
		GLushort *otherUShortImagePtr = (GLushort*)image._imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ushortImagePtr++ = *otherUShortImagePtr++;
				}
				ushortImagePtr += _extraRowWidth;
				otherUShortImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ushortImagePtr++ = *otherUShortImagePtr++;
					*ushortImagePtr++ = *otherUShortImagePtr++;
					*ushortImagePtr++ = *otherUShortImagePtr++;
				}
				ushortImagePtr += _extraRowWidth;
				otherUShortImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*ushortImagePtr++ = *otherUShortImagePtr++;
					*ushortImagePtr++ = *otherUShortImagePtr++;
					*ushortImagePtr++ = *otherUShortImagePtr++;
					*ushortImagePtr++ = *otherUShortImagePtr++;
				}
				ushortImagePtr += _extraRowWidth;
				otherUShortImagePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_INT) {
		GLint *intImagePtr = (GLint*)_imageData;
		GLint *otherIntImagePtr = (GLint*)image._imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*intImagePtr++ = *otherIntImagePtr++;
				}
				intImagePtr += _extraRowWidth;
				otherIntImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*intImagePtr++ = *otherIntImagePtr++;
					*intImagePtr++ = *otherIntImagePtr++;
					*intImagePtr++ = *otherIntImagePtr++;
				}
				intImagePtr += _extraRowWidth;
				otherIntImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*intImagePtr++ = *otherIntImagePtr++;
					*intImagePtr++ = *otherIntImagePtr++;
					*intImagePtr++ = *otherIntImagePtr++;
					*intImagePtr++ = *otherIntImagePtr++;
				}
				intImagePtr += _extraRowWidth;
				otherIntImagePtr += _extraRowWidth;
			}
		}
	}
	else if (_imageDataType == GL_UNSIGNED_INT) {
		GLuint *uintImagePtr = (GLuint*)_imageData;
		GLuint *otherUIntImagePtr = (GLuint*)image._imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*uintImagePtr++ = *otherUIntImagePtr++;
				}
				uintImagePtr += _extraRowWidth;
				otherUIntImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*uintImagePtr++ = *otherUIntImagePtr++;
					*uintImagePtr++ = *otherUIntImagePtr++;
					*uintImagePtr++ = *otherUIntImagePtr++;
				}
				uintImagePtr += _extraRowWidth;
				otherUIntImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*uintImagePtr++ = *otherUIntImagePtr++;
					*uintImagePtr++ = *otherUIntImagePtr++;
					*uintImagePtr++ = *otherUIntImagePtr++;
					*uintImagePtr++ = *otherUIntImagePtr++;
				}
				uintImagePtr += _extraRowWidth;
				otherUIntImagePtr += _extraRowWidth;
			}
		}
	}
	else {          // _imageDataType == GL_FLOAT
		GLfloat *floatImagePtr = (GLfloat*)_imageData;
		GLfloat *otherFloatImagePtr = (GLfloat*)image._imageData;
	
		if (_imageType == GL_COLOR_INDEX) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*floatImagePtr++ = *otherFloatImagePtr++;
				}
				floatImagePtr += _extraRowWidth;
				otherFloatImagePtr += _extraRowWidth;
			}
		}
		else if (_imageType == GL_RGB) {
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*floatImagePtr++ = *otherFloatImagePtr++;
					*floatImagePtr++ = *otherFloatImagePtr++;
					*floatImagePtr++ = *otherFloatImagePtr++;
				}
				floatImagePtr += _extraRowWidth;
				otherFloatImagePtr += _extraRowWidth;
			}
		}
		else {       // _imageType == GL_RGBA
			for (i=_height; i > 0; i--) {
				for (j=_width; j > 0; j--) {
					*floatImagePtr++ = *otherFloatImagePtr++;
					*floatImagePtr++ = *otherFloatImagePtr++;
					*floatImagePtr++ = *otherFloatImagePtr++;
					*floatImagePtr++ = *otherFloatImagePtr++;
				}
				floatImagePtr += _extraRowWidth;
				otherFloatImagePtr += _extraRowWidth;
			}
		}
	}




	//
	// copy the colormap information
	//
	_colormapSize = image._colormapSize;
	_colormapDataType = image._colormapDataType;
	_allocateColormapData();


	//
	// duplicate a *LOT* of code to make operations go faster. Also, the large
	// amount of code duplication is necessary due to the different data types
	// that an image can be.
	//

	if (_colormapDataType == GL_BYTE) {
		GLbyte *byteColormapPtr = (GLbyte*)_colormapData;
		GLbyte *otherByteColormapPtr = (GLbyte*)image._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*byteColormapPtr++ = *otherByteColormapPtr++;
			*byteColormapPtr++ = *otherByteColormapPtr++;
			*byteColormapPtr++ = *otherByteColormapPtr++;
		}
	}
	else if (_colormapDataType == GL_UNSIGNED_BYTE) {
		GLubyte *ubyteColormapPtr = (GLubyte*)_colormapData;
		GLubyte *otherUByteColormapPtr = (GLubyte*)image._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*ubyteColormapPtr++ = *otherUByteColormapPtr++;
			*ubyteColormapPtr++ = *otherUByteColormapPtr++;
			*ubyteColormapPtr++ = *otherUByteColormapPtr++;
		}
	}
	else if (_colormapDataType == GL_SHORT) {
		GLshort *shortColormapPtr = (GLshort*)_colormapData;
		GLshort *otherShortColormapPtr = (GLshort*)image._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*shortColormapPtr++ = *otherShortColormapPtr++;
			*shortColormapPtr++ = *otherShortColormapPtr++;
			*shortColormapPtr++ = *otherShortColormapPtr++;
		}
	}
	else if (_colormapDataType == GL_UNSIGNED_SHORT) {
		GLushort *ushortColormapPtr = (GLushort*)_colormapData;
		GLushort *otherUShortColormapPtr = (GLushort*)image._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*ushortColormapPtr++ = *otherUShortColormapPtr++;
			*ushortColormapPtr++ = *otherUShortColormapPtr++;
			*ushortColormapPtr++ = *otherUShortColormapPtr++;
		}
	}
	else if (_colormapDataType == GL_INT) {
		GLint *intColormapPtr = (GLint*)_colormapData;
		GLint *otherIntColormapPtr = (GLint*)image._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*intColormapPtr++ = *otherIntColormapPtr++;
			*intColormapPtr++ = *otherIntColormapPtr++;
			*intColormapPtr++ = *otherIntColormapPtr++;
		}
	}
	else if (_colormapDataType == GL_UNSIGNED_INT) {
		GLuint *uintColormapPtr = (GLuint*)_colormapData;
		GLuint *otherUIntColormapPtr = (GLuint*)image._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*uintColormapPtr++ = *otherUIntColormapPtr++;
			*uintColormapPtr++ = *otherUIntColormapPtr++;
			*uintColormapPtr++ = *otherUIntColormapPtr++;
		}
	}
	else {          // _colormapDataType == GL_FLOAT
		GLfloat *floatColormapPtr = (GLfloat*)_colormapData;
		GLfloat *otherFloatColormapPtr = (GLfloat*)image._colormapData;

		for (i=_colormapSize; i > 0; i--) {
			*floatColormapPtr++ = *otherFloatColormapPtr++;
			*floatColormapPtr++ = *otherFloatColormapPtr++;
			*floatColormapPtr++ = *otherFloatColormapPtr++;
		}
	}



	return *this;
}




