
#include <math.h>
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "SceneIO.h"
#include "RenderPreview.h"
#include "RenderFaceSet.h"
#include "RenderQuadMesh.h"
#include "RenderTriMesh.h"

void RenderPreview( SceneIO *gscene )
{
	ObjIO* obj = gscene->objects;

	double angle = 50.0;
	double aspect = 1.0;
	double nearclip = 0.1;
	double farclip = 10000.0;

	glEnable(GL_DEPTH_TEST);
	glDepthMask(GL_TRUE);
	glClearDepth(1.0);

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glEnable(GL_LIGHTING);
	glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, 1 );


	glEnable(GL_BLEND);

	//
	// A over B
	//
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);


	//
	//	Set up Lights
	//
	LightIO *light = gscene->lights;
	float pos[4];
	float light_color[4];
	int lcnt = 0;;
	while( light != NULL )
	{
		glEnable(GL_LIGHT0 + lcnt);

		switch( light->type )
		{
		case POINT_LIGHT:
#if 0
			// For some reason this doesn't work on the NT machines??
			fprintf(stderr,"Adding positional light\n");
			pos[0] = light->position[0];
			pos[1] = light->position[1];
			pos[2] = light->position[2];
			pos[3] = 1.0f;

			light_color[3] = 1.0f;
			light_color[0] = light->color[0];
			light_color[1] = light->color[1];
			light_color[2] = light->color[2];
			glLightfv( GL_LIGHT0+lcnt, GL_POSITION, pos );
			glLightfv( GL_LIGHT0+lcnt, GL_DIFFUSE, light_color );
#endif
			break;

		case DIRECTIONAL_LIGHT:
			fprintf(stderr,"Adding directional light\n");
			pos[0] = -light->direction[0];
			pos[1] = -light->direction[1];
			pos[2] = -light->direction[2];
			pos[3] = 0;
			light_color[3] = 1.0f;
			light_color[0] = light->color[0];
			light_color[1] = light->color[1];
			light_color[2] = light->color[2];
			glLightfv( GL_LIGHT0+lcnt, GL_POSITION, pos );
			glLightfv( GL_LIGHT0+lcnt, GL_DIFFUSE, light_color );
			break;

		case SPOT_LIGHT:
			fprintf(stderr,"Spot lights not yet supported\n");
			break;

		}
		light = light->next;
		lcnt++;
	}

	glEnable(GL_NORMALIZE);

	//
	//	Set up Camera
	//
#if(1)
		glMatrixMode( GL_PROJECTION );
		glLoadIdentity();
		gluPerspective( gscene->camera->verticalFOV*180.0/3.1415926,
						aspect,
						nearclip,
						farclip );

		glMatrixMode( GL_MODELVIEW );
		glLoadIdentity();

		gluLookAt(	gscene->camera->position[0],
					gscene->camera->position[1],
					gscene->camera->position[2],

					gscene->camera->position[0] + gscene->camera->viewDirection[0],
					gscene->camera->position[1] + gscene->camera->viewDirection[1],
					gscene->camera->position[2] + gscene->camera->viewDirection[2],

					gscene->camera->orthoUp[0],
					gscene->camera->orthoUp[1],
					gscene->camera->orthoUp[2]
					);
		
#else
		glMatrixMode( GL_PROJECTION );
		glLoadIdentity();
		gluPerspective( angle,
						aspect,
						nearclip,
						farclip );

		glMatrixMode( GL_MODELVIEW );
		glLoadIdentity();

		gluLookAt( 0,0,20, 0,0,19, 0,1,0 );
#endif

	//
	//	Render Geometry
	//
	while( obj != NULL )
	{
		PolySetIO *pset = NULL;
		SphereIO *sphere = NULL;
		GLUquadricObj *quad = NULL;
		
		switch( obj->type )
		{
		case SPHERE_OBJ:
			sphere = (SphereIO *)obj->data;

			quad = gluNewQuadric();

			glPushMatrix();
			glTranslatef( sphere->origin[0], sphere->origin[1], sphere->origin[2] );

			if (obj->material) {
				float color[4];

				color[3] = 1.0f - obj->material->ktran;
				
				color[0] = obj->material->ambColor[0];
				color[1] = obj->material->ambColor[1];
				color[2] = obj->material->ambColor[2];

				glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, color);

				color[0] = obj->material->diffColor[0];
				color[1] = obj->material->diffColor[1];
				color[2] = obj->material->diffColor[2];
				glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, color);

				color[0] = obj->material->specColor[0];
				color[1] = obj->material->specColor[1];
				color[2] = obj->material->specColor[2];
				glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, color);

				glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, obj->material->shininess);

				color[0] = obj->material->emissColor[0];
				color[1] = obj->material->emissColor[1];
				color[2] = obj->material->emissColor[2];
				glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, color);
			}

			gluSphere( quad,  sphere->radius, 180, 90 );
			glPopMatrix();
			gluDeleteQuadric( quad );
			break;

		case POLYSET_OBJ:
			pset = (PolySetIO *)obj->data;
			if( pset->type == POLYSET_TRI_MESH ) {
				if( pset->normType == PER_FACE_NORMAL ) {
					if( pset->materialBinding == PER_OBJECT_MATERIAL )
					{
						glPushMatrix();
						RenderTriMesh_POM_PFN( pset, obj->material, obj->numMaterials );
						glPopMatrix();
					}
					else {            // PER_VERTEX_MATERIAL
						glPushMatrix();
						RenderTriMesh_PVM_PFN( pset, obj->material, obj->numMaterials );
						glPopMatrix();
					}
				}
				else if ( pset->normType == PER_VERTEX_NORMAL ) {
					if( pset->materialBinding == PER_OBJECT_MATERIAL )
					{
						glPushMatrix();
						RenderTriMesh_POM_PVN( pset, obj->material, obj->numMaterials );
						glPopMatrix();
					}
					else {            // PER_VERTEX_MATERIAL
						glPushMatrix();
						RenderTriMesh_PVM_PVN( pset, obj->material, obj->numMaterials );
						glPopMatrix();
					}
				}
			}
			else if (pset->type == POLYSET_QUAD_MESH) {
				if( pset->normType == PER_FACE_NORMAL ) {
					if( pset->materialBinding == PER_OBJECT_MATERIAL )
					{
						glPushMatrix();
						RenderQuadMesh_POM_PFN( pset, obj->material, obj->numMaterials );
						glPopMatrix();
					}
					else {            // PER_VERTEX_MATERIAL
						glPushMatrix();
						RenderQuadMesh_PVM_PFN( pset, obj->material, obj->numMaterials );
						glPopMatrix();
					}
				}
				else if ( pset->normType == PER_VERTEX_NORMAL ) {
					if( pset->materialBinding == PER_OBJECT_MATERIAL )
					{
						glPushMatrix();
						RenderQuadMesh_POM_PVN( pset, obj->material, obj->numMaterials );
						glPopMatrix();
					}
					else {            // PER_VERTEX_MATERIAL
						glPushMatrix();
						RenderQuadMesh_PVM_PVN( pset, obj->material, obj->numMaterials );
						glPopMatrix();
					}
				}
			}
			else if (pset->type == POLYSET_FACE_SET) {
				if( pset->normType == PER_FACE_NORMAL ) {
					if( pset->materialBinding == PER_OBJECT_MATERIAL )
					{
						glPushMatrix();
						RenderFaceSet_POM_PFN( pset, obj->material, obj->numMaterials );
						glPopMatrix();
					}
					else {            // PER_VERTEX_MATERIAL
						glPushMatrix();
						RenderFaceSet_PVM_PFN( pset, obj->material, obj->numMaterials );
						glPopMatrix();
					}
				}
				else if ( pset->normType == PER_VERTEX_NORMAL ) {
					if( pset->materialBinding == PER_OBJECT_MATERIAL )
					{
						glPushMatrix();
						RenderFaceSet_POM_PVN( pset, obj->material, obj->numMaterials );
						glPopMatrix();
					}
					else {            // PER_VERTEX_MATERIAL
						glPushMatrix();
						RenderFaceSet_PVM_PVN( pset, obj->material, obj->numMaterials );
						glPopMatrix();
					}
				}
			}
			break;

		};
		obj = obj->next;
	}
}
