// SceneView.cpp : implementation file
//

#include "stdafx.h"
#include "RayTracer.h"
#include "RayTracerDoc.h"
#include "SceneView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSceneView

IMPLEMENT_DYNCREATE(CSceneView, CView)

CSceneView::CSceneView() :
	m_hRC(0), m_pDC(0), m_cx(400), m_cy(400), m_ErrorString(_ErrorStrings[0])
{
	m_pScene=NULL;
}

CSceneView::~CSceneView()
{
}


BEGIN_MESSAGE_MAP(CSceneView, CView)
	//{{AFX_MSG_MAP(CSceneView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSceneView drawing

void CSceneView::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code here
	// Clear the buffers
	InitializeOpenGL();
	SetupViewport(m_cx,m_cy);
	GLdouble aspect_ratio;
	aspect_ratio = (GLdouble)m_cx/(GLdouble)m_cy;

	//select the projection matrix and clear it.
	::glMatrixMode(GL_PROJECTION);
	::glLoadIdentity();

	//select the viewing volume
	SetupViewingFrustum( aspect_ratio );

	//switch back to the modelview matrix and clear it. 
	::glMatrixMode( GL_MODELVIEW );
	::glLoadIdentity();

	// now perform any viewing transformations.
	SetupViewingTransform();
	
	::glDrawBuffer(GL_BACK);
	::glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT  );

	if(m_pScene!=NULL)
	{
		RenderPreview( m_pScene );
	}

	if( FALSE == ::SwapBuffers( m_pDC->GetSafeHdc()))
	{
		SetError(7);
	}
	::glFinish();
}

/////////////////////////////////////////////////////////////////////////////
// CSceneView diagnostics

#ifdef _DEBUG
void CSceneView::AssertValid() const
{
	CView::AssertValid();
}

void CSceneView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSceneView message handlers

BOOL CSceneView::PreCreateWindow(CREATESTRUCT& cs) 
{
	// TODO: Add your specialized code here and/or call the base class
	cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	return CView::PreCreateWindow(cs);
}

int CSceneView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	InitializeOpenGL();
	return 0;
}

void CSceneView::OnDestroy() 
{
	CView::OnDestroy();
	
	// TODO: Add your message handler code here
	if ( FALSE == ::wglMakeCurrent( 0,0 ))
	{
		SetError(2);
	}

	if ( FALSE == ::wglDeleteContext( m_hRC ))
	{
		SetError(6);
	}

	if ( m_pDC )
	{
		delete m_pDC;
	}
}

BOOL CSceneView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	//return CView::OnEraseBkgnd(pDC);

	return TRUE;
}

void CSceneView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	m_cx = cx;
	m_cy = cy;

	// TODO: Add your message handler code here
	GLdouble aspect_ratio; // width/height ratio

	if( 0>= cx || 0>=cy )
	{
		return;
	}

	SetupViewport( cx, cy );

	// compute the aspect ratio
	// this will keep all dimension scales equal.
	aspect_ratio = (GLdouble)cx/(GLdouble)cy;

	//select the projection matrix and clear it.
	::glMatrixMode(GL_PROJECTION);
	::glLoadIdentity();

	//select the viewing volume
	SetupViewingFrustum( aspect_ratio );

	//switch back to the modelview matrix and clear it. 
	::glMatrixMode( GL_MODELVIEW );
	::glLoadIdentity();

	// now perform any viewing transformations.
	SetupViewingTransform();
}

//These are the general errors that can occur when setting up an openGL window.
const char* const CSceneView::_ErrorStrings[] = {
	{"No Error"}, //0
	{"Unable to get a DC"},         //1
	{"Choose Pixel Format failed"}, //2
	{"Select Pixel Format failed"}, //3
	{"wglCreateContext failed"},    //4
	{"wglMakeCurrent failed"},      //5
	{"wglDeleteCintext failed"},    //6
	{"Swap Buffers failed"},        //7
};

void CSceneView::SetError( int e )
{
	//if there was no previous error then save this one.
	if(_ErrorStrings[0] == m_ErrorString )
	{
		m_ErrorString = _ErrorStrings[e];
	}
}

BOOL CSceneView::InitializeOpenGL()
{
	m_pDC = new CClientDC(this);

	if( NULL == m_pDC) //failure to get DC
	{
		SetError(1);
		return FALSE;
	}

	if( !SetupPixelFormat() )
	{
		return FALSE;
	}

	if( 0== (m_hRC = ::wglCreateContext( m_pDC->GetSafeHdc())))
	{
		SetError(4);
		return FALSE;
	}

	if( FALSE == ::wglMakeCurrent( m_pDC->GetSafeHdc(), m_hRC))
	{
		SetError(5);
		return FALSE;
	}

	//Specify black as clear color
	::glClearColor( 0.0f, 0.0f, 0.0f, 0.0f);

	//Specify the back of the buffer as clear depth
	::glClearDepth(1.0f);

	//Enable depth testing
	::glEnable( GL_DEPTH_TEST );

	return TRUE;
}

BOOL CSceneView::SetupPixelFormat()
{
	PIXELFORMATDESCRIPTOR pfd = 
	{
		sizeof(PIXELFORMATDESCRIPTOR),  // size of the pfd
		1,								// version number
		PFD_DRAW_TO_WINDOW |			// support window 
		PFD_SUPPORT_OPENGL |			// support OpenGL
		PFD_DOUBLEBUFFER,				// double buffered
		PFD_TYPE_RGBA,					// RGBA type
		24,								// 24 bit color depth
		0,0,0,0,0,0,					// color bits ignored
		0,								// no alpha buffer
		0,								// shift bit ignored
		0,								// no accumulation buffer
		0,0,0,0,						// accum bits ignored
		16,								// 16-bit z-buffer
		0,								// no stencil buffer
		0,								// no auxiliary buffer
		PFD_MAIN_PLANE,					// main layer
		0,								// reserved 
		0,0,0							// layer masks ignored
	};

	int pixelformat;

	if ( 0 == (pixelformat = ::ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)))
	{
		SetError(2);
		return FALSE;
	}

	if (FALSE == ::SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd))
	{
		SetError(3);
		return FALSE;
	}

	return TRUE;
}

BOOL CSceneView::SetupViewport( int cx, int cy )
{
	// select the full client area
	glViewport(0,0,cx,cy);

	return TRUE;
}

BOOL CSceneView::SetupViewingFrustum(GLdouble aspect_ratio)
{
	// select a default viewing volume.
	gluPerspective(40.0f, aspect_ratio, .1f, 20.0f);

	return TRUE;
}

BOOL CSceneView::SetupViewingTransform()
{
	// select a default viewing transform
	// of a 20 degree rotation about the X axis,
	// then a -5 unit transformation along the Z
	::glTranslatef( 0.0f, 0.0f, -5.0f );
	::glRotatef( 20.0f, 1.0f, 0.0f, 0.0f );

	return TRUE;
}


void CSceneView::Openscene() 
{
	// TODO: Add your command handler code here
	CRayTracerDoc* pDoc = (CRayTracerDoc*) GetDocument();

	//Open the scene
	pDoc->OpenScene();

	//Get the scene from the document class so it can be drawn.
	m_pScene = pDoc->RetrieveScene();
	
	// Redraw the window
	OnDraw(m_pDC);
}

