// TraceView.cpp : implementation file
//

#include "stdafx.h"
#include "math.h"
#include "RayTracer.h"
#include "RayTracerDoc.h"


#include "TraceView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTraceView

IMPLEMENT_DYNCREATE(CTraceView, CView)



CTraceView::CTraceView() :
	m_hRC(0), m_pDC(0), m_cx(400), m_cy(400),m_ErrorString(_ErrorStrings[0])
{
	m_pScene = NULL;
	raytraceImage.setSize(m_cx,m_cy);

	//Set up the raytracing parameters
		//Set the RayTracing Parameters
	m_RTparams.rayeps    = 0.0005f;
	m_RTparams.minweight = 0.05f;
	m_RTparams.maxlevel  = 5;

}

CTraceView::~CTraceView()
{
}


BEGIN_MESSAGE_MAP(CTraceView, CView)
	//{{AFX_MSG_MAP(CTraceView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTraceView drawing

void CTraceView::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();
	// TODO: add draw code here
	// Clear the buffers
	//::glDrawBuffer( GL_FRONT );
	InitializeOpenGL();
	SetupViewport(m_cx,m_cy);

	//select the projection matrix and clear it.
	::glMatrixMode(GL_PROJECTION);
	::glLoadIdentity();

	//select the viewing volume
	SetupViewingFrustum( /*(GLdouble)aspect_ratio*/(GLdouble)m_cx, (GLdouble)m_cy );

	//switch back to the modelview matrix and clear it. 
	::glMatrixMode( GL_MODELVIEW );
	::glLoadIdentity();

	// now perform any viewing transformations.
	SetupViewingTransform();
	//::glDrawBuffer(GL_BACK);
	::glDrawBuffer(GL_FRONT);
	::glClear( GL_COLOR_BUFFER_BIT ); //| GL_DEPTH_BUFFER_BIT );
	
	if(m_pScene!=NULL)
	{
		raytraceImage.draw();
	}


#if 0
	//StockScene();
	glShadeModel(GL_SMOOTH);
	glBegin(GL_TRIANGLES);
		glColor3f(1.0f, 0.0f, 0.0f);
		glVertex2i(0, 0);
		glColor3f(0.0f, 1.0f, 0.0f);
		glVertex2i(m_cx, 0);
		//glVertex2i(10, 0);
		glColor3f(0.0f, 0.0f, 1.0f);
		glVertex2i(m_cx/2, m_cy);
		//glVertex2i(5, 10);
	glEnd();

	if( FALSE == ::SwapBuffers( m_pDC->GetSafeHdc()))
	{
		SetError(7);
	}
#endif

	::glFinish();


}

/////////////////////////////////////////////////////////////////////////////
// CTraceView diagnostics

#ifdef _DEBUG
void CTraceView::AssertValid() const
{
	CView::AssertValid();
}

void CTraceView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTraceView message handlers

int CTraceView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	InitializeOpenGL();
	return 0;
}

void CTraceView::OnDestroy() 
{
	CView::OnDestroy();
	
	// TODO: Add your message handler code here
	if ( FALSE == ::wglMakeCurrent( 0,0 ))
	{
		SetError(2);
	}

	if ( FALSE == ::wglDeleteContext( m_hRC ))
	{
		SetError(6);
	}

	if ( m_pDC )
	{
		delete m_pDC;
	}
}

BOOL CTraceView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	//return CView::OnEraseBkgnd(pDC);

	return TRUE;
}

void CTraceView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	GLdouble aspect_ratio; // width/height ratio

	m_cx = cx;
	m_cy = cy;

	if( 0>= cx || 0>=cy )
	{
		return;
	}

	SetupViewport( cx, cy );

	// compute the aspect ratio
	// this will keep all dimension scales equal.
	aspect_ratio = (GLdouble)cx/(GLdouble)cy;

	//select the projection matrix and clear it.
	::glMatrixMode(GL_PROJECTION);
	::glLoadIdentity();

	//select the viewing volume
	SetupViewingFrustum( /*aspect_ratio*/cx, cy );

	//switch back to the modelview matrix and clear it. 
	::glMatrixMode( GL_MODELVIEW );
	::glLoadIdentity();

	// now perform any viewing transformations.
	SetupViewingTransform();

	//Resize the raytraced image. 
	raytraceImage.setXScale(m_cx/(float)raytraceImage.getWidth());
	raytraceImage.setYScale(m_cy/(float)raytraceImage.getHeight());
}

BOOL CTraceView::PreCreateWindow(CREATESTRUCT& cs) 
{
	// TODO: Add your specialized code here and/or call the base class
	cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	return CView::PreCreateWindow(cs);
}

//These are the general errors that can occur when setting up an openGL window.
const char* const CTraceView::_ErrorStrings[] = {
	{"No Error"}, //0
	{"Unable to get a DC"},         //1
	{"Choose Pixel Format failed"}, //2
	{"Select Pixel Format failed"}, //3
	{"wglCreateContext failed"},    //4
	{"wglMakeCurrent failed"},      //5
	{"wglDeleteCintext failed"},    //6
	{"Swap Buffers failed"},        //7
};

void CTraceView::SetError( int e )
{
	//if there was no previous error then save this one.
	if(_ErrorStrings[0] == m_ErrorString )
	{
		m_ErrorString = _ErrorStrings[e];
	}
}

BOOL CTraceView::InitializeOpenGL()
{
	m_pDC = new CClientDC(this);

	if( NULL == m_pDC) //failure to get DC
	{
		SetError(1);
		return FALSE;
	}

	if( !SetupPixelFormat() )
	{
		return FALSE;
	}

	if( 0== (m_hRC = ::wglCreateContext( m_pDC->GetSafeHdc())))
	{
		SetError(4);
		return FALSE;
	}

	if( FALSE == ::wglMakeCurrent( m_pDC->GetSafeHdc(), m_hRC))
	{
		SetError(5);
		return FALSE;
	}

	//Specify black as clear color
	::glClearColor( 0.0f, 0.0f, 0.0f, 0.0f);

	//Specify the back of the buffer as clear depth
	::glClearDepth(1.0f);

	//Enable depth testing
	//::glEnable( GL_DEPTH_TEST );

	::glDisable(GL_DEPTH_TEST);

	return TRUE;
}

BOOL CTraceView::SetupPixelFormat()
{
	PIXELFORMATDESCRIPTOR pfd = 
	{
		sizeof(PIXELFORMATDESCRIPTOR),  // size of the pfd
		1,								// version number
		PFD_DRAW_TO_WINDOW |			// support window 
		PFD_SUPPORT_OPENGL, // |			// support OpenGL
		//PFD_DOUBLEBUFFER,				// double buffered
		PFD_TYPE_RGBA,					// RGBA type
		24,								// 24 bit color depth
		0,0,0,0,0,0,					// color bits ignored
		0,								// no alpha buffer
		0,								// shift bit ignored
		0,								// no accumulation buffer
		0,0,0,0,						// accum bits ignored
		16,								// 16-bit z-buffer
		0,								// no stencil buffer
		0,								// no auxiliary buffer
		PFD_MAIN_PLANE,					// main layer
		0,								// reserved 
		0,0,0							// layer masks ignored
	};

	int pixelformat;

	if ( 0 == (pixelformat = ::ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)))
	{
		SetError(2);
		return FALSE;
	}

	if (FALSE == ::SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd))
	{
		SetError(3);
		return FALSE;
	}

	return TRUE;
}

BOOL CTraceView::SetupViewport( int cx, int cy )
{
	// select the full client area
	glViewport(0,0,cx,cy);

	return TRUE;
}

BOOL CTraceView::SetupViewingFrustum(GLdouble width, GLdouble height)
{
	// select a default viewing volume.
	glOrtho(0.0, width, 0.0, height, 1.0, -1.0);
	//gluPerspective(40.0f, /*width*/1, .1f, 20.0f);

	return TRUE;
}

BOOL CTraceView::SetupViewingTransform()
{
	// select a default viewing transform
	// of a 20 degree rotation about the X axis,
	// then a -5 unit transformation along the Z
	//::glTranslatef( 0.0f, 0.0f, -1.0f );
	//::glRotatef( 20.0f, 1.0f, 0.0f, 0.0f );

	return TRUE;
}

//These will be taken out in my version, this is just a template
BOOL CTraceView::RenderScene()
{
	::glRotatef( 90.0f, 1.0f, 0.0f, 0.0f );
	::glColor3f( 1.0f, 0.0f, 0.0f );
	::auxWireSphere( .5 );
	::glColor3f( 0.5f, 0.5f, 1.0f );
	::auxWireCube (1.0);
	return TRUE;
}

void CTraceView::StockScene()
{
	// define all vertices X Y Z
	GLfloat v0[3], v1[3], v2[3], v3[3], delta;
	int color = 0;

	delta  = 0.5f;

	//define the two colors
	GLfloat color1[3] = { 0.9f, 0.9f, 0.9f };
	GLfloat color2[3] = { 0.05f, 0.05f, 0.05f };

	v0[1] = v1[1] = v2[1] = v3[1] = 0.0f;

	::glBegin( GL_QUADS );

	for( int x=-5; x<=5; x++)
	{
		for( int z = -5; z<=5; z++)
		{
			::glColor3fv( (color++)%2 ? color1 : color2 );

			v0[0] = 0.0f + delta*z;
			v0[2] = 0.0f + delta*x;

			v1[0] = v0[0] + delta;
			v1[2] = v0[2];

			v2[0] = v0[0] + delta;
			v2[2] = v0[2] + delta;

			v3[0] = v0[0];
			v3[2] = v0[2] + delta;

			::glVertex3fv( v0 );
			::glVertex3fv( v1 );
			::glVertex3fv( v2 );
			::glVertex3fv( v3 );
		}
	}

	::glEnd();
}

void CTraceView::RenderScreen(SceneIO *scene, Image &image )
{
	Vector3f E;
	Vector3f G;
	Vector3f U;
	Vector3f M;
	Vector3f H;
	Vector3f V;
	Vector3f A;
	Vector3f B;
	Vector3f P;
	float theta, phi;
	int ix,iy;
	float sx, sy;

	Vector3f temp1, temp2, temp3;
	Vector3f color;
	Ray ray;
	int r,g,b;
	int width  = image.getWidth();
	int height = image.getHeight();

	/* copy variables from the input scene */
	Vector3f_copy  (scene->camera->position, E);
	Vector3f_copy  (scene->camera->viewDirection, G);
	Vector3f_scale (scene->camera->focalDistance, G);
	Vector3f_copy  (scene->camera->orthoUp, U);
	theta = phi = scene->camera->verticalFOV / 2;

	/* calculate up and right vectors */
	Vector3f_cross(G, U, A);
	Vector3f_cross(A, G, B);

	/* calculate midpoint of screen */
	Vector3f_add(E, G, M);  

	/* calculate viewplane spanners */
	Vector3f_normalize (A);
	Vector3f_scale2 (Vector3f_length(G)*tanf(theta), A, H);

	Vector3f_normalize(B);
	Vector3f_scale2 (Vector3f_length(G)*tan(phi), B, V);


	//
	// loop through vertical scanlines
	//
	for (iy=0; iy<height; iy++)
	{
		sy = (float)iy / height;

		// loop through horizontal pixels
		for (ix=0; ix<width; ix++)
		{
			sx = (float)ix / width;

			// calculate position of pixel
			Vector3f_scale2 (2*sx - 1, H, temp1);
			Vector3f_scale2 (2*sy - 1, V, temp2);
			Vector3f_add (M, temp1, temp3);
			Vector3f_add (temp3, temp2, P);	  

			// calculate ray direction at eyepoint
			Vector3f_copy (E, ray.P);
			Vector3f_sub (P, E, ray.D);
			Vector3f_normalize(ray.D);

			// trace the ray and set the pixel color
			TraceRay(scene, 0, 1., &ray, color, &m_RTparams);
   
			r = (int)(color[0]*255);
			g = (int)(color[1]*255);
			b = (int)(color[2]*255);

			//
			//	This is a different Color, a class from the
			//	image library.
			//
			Color c;
			c.setRGB(r,g,b);
			//unsigned char aColor = (unsigned char)((float)ix / (float)width * 256.0f);
			//c.setRGB(aColor, 0, 0);
			image[iy*width + ix] = c;
		}


		if (iy % 30 == 0) {
			//image.draw(); //0, 0, width, iy);
			//glFinish();
			OnDraw(m_pDC);
		}
	}

}


void CTraceView::RaytraceScene()
{	
	CRayTracerDoc* pDoc = (CRayTracerDoc*) GetDocument();

	// Retrieve the scene
	m_pScene = pDoc->RetrieveScene();

	if(m_pScene != NULL)
	{
       
		raytraceImage.setSize(m_cx, m_cy);
		raytraceImage.setScale(1.0f);
		
	   //Render the scene
	   RenderScreen( m_pScene, raytraceImage );

	   //Save the image
	   imageToSave = raytraceImage;
	   
	   // Redraw the window
	   OnDraw(m_pDC);
	}	
}

void CTraceView::SetDepthAndHeight(int depth, float weight)
{
	m_RTparams.minweight = weight;
	m_RTparams.maxlevel  = depth;
}

void CTraceView::FileSave(CString filename)
{
	imageToSave.convertToRGB();

	imageToSave.write(filename, ImageFile::BMP_FILE);
}