/***************************************************************************
 *
 * File:      Vector3f.h
 *
 * Description: 
 *   Routines for length 3 vectors 
 *
 ***************************************************************************/

#ifndef _Vector3f_h_
#define _Vector3f_h_

	/* Definitions of fundamental types a la Hecbert in	*/
	/*   Glassner''s _Introduction_to_Ray_Tracing_	*/

typedef float Vector3f[3];	    /* Vector in 3-space (nx, ny, nz)			*/
typedef Vector3f Point3f;	    /* Point in 3-space (x, y, z)				*/
typedef Vector3f Color3f;	    /* Float rep. of a color (r, g, b) where	*/
								/*   0.0 <= r,g,b <= 1.0.					*/

/* Copies a vector. vout = vin */
void Vector3f_copy(float *vin, float *vout);

/* Computes vout = v1 + v2 */
void Vector3f_add(float *v1, float *v2, float *vout);

/* Computes vout = v1 - v2 */
void Vector3f_sub(float *v1, float *v2, float *vout);

/* Normalizes v to length 1 */
void Vector3f_normalize(float *v);

/* Scales all components of v by s */
void Vector3f_scale(float s, float *v);

/* Computes out = s*in */
void Vector3f_scale2(float s, float *in, float *out);

/* Returns the length of v */
float Vector3f_length(float *v);

/* Returns the dot product of v1 and v2 */
float Vector3f_dot(float *v1, float *v2);

/* Sets v to (v1,v2,v3) */
void Vector3f_set(float *v, float v1, float v2, float v3);

/* Sets vout to the component-by-component multiply of v1 and v2.
   ie.  vout[0] = v1[0]*v2[0], etc. */
void Vector3f_comp_mult(float *v1, float *v2, float *vout);

/* Computes vout = v1 X v2 */
void Vector3f_cross(float *v1, float *v2, float *vout);

#endif /* _Vector3f_h_ */




